#ifndef CHEADER_H_INCLUDED
#define CHEADER_H_INCLUDED

#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/config_api.h>
#include <libmemcached/memcached.h>
#include <string.h>
#include <tpl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <mysql/mysql.h>
#include <regex.h>
#include <sys/types.h>
#include <errno.h>
#include <signal.h>
#include <sys/wait.h>
#include <math.h>
#include <ctype.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * OpMon SNMP check special scripts
 * include
 */
#define DBOPMON                     "opmon4"
#define DBCONF                      "/usr/local/opmon/etc/db.php"
#define TRUE                        1
#define FALSE                       0
#define OK                          0
#define WARNING                     1
#define CRITICAL                    2
#define UNKNOWN                     3
#define ERROR                       -2
#define MAX_CMD_ARGS                4096
#define OP_PLUGIN_BUFFERSIZE        98304
#define OP_MAX_OUTPUT               65536
#define MAX_PLUGIN_OUTPUT_LENGTH    8192


/**
 * database snmp_community table
 * fields index
 */
#define FIELD_COMMUNITY_ID 0
#define FIELD_SNMP_VERSION 1
#define FIELD_COMMUNITY 2
#define FIELD_AUTHENTICATION_PROTOCOL 3
#define FIELD_AUTHENTICATION_PASSPHRASE 4
#define FIELD_SECURITY_LEVEL 5
#define FIELD_CONTEXT_NAME 6
#define FIELD_SECURITY_USERNAME 7
#define FIELD_PRIVACY_PROTOCOL 8
#define FIELD_PRIVACY_PROTOCOL_PASSPHRASE 9
#define FIELD_DESTINATION_ENGINE 10
#define FIELD_ACTIVE 11

/*
 * type of baselines
 */
#define BASELINE_TYPE_DAY 0
#define BASELINE_TYPE_WEEK 1
#define BASELINE_TYPE_MONTH 2
#define BASELINE_TYPE_CALENDAR 3

#define BASELINE_ALARM_DISABLED 0
#define BASELINE_ALARM_UP 1
#define BASELINE_ALARM_DOWN 2
#define BASELINE_ALARM_BOTH 3

/**
 * database baseline table
 * fields index
 */
#define FIELD_BASELINE_ID 0
#define FIELD_BASELINE_NAME 1
#define FIELD_BASELINE_DESCRIPTION 2
#define FIELD_BASELINE_TYPE 3
#define FIELD_BASELINE_SAMPLES 4
#define FIELD_BASELINE_ALARM 5
#define FIELD_BASELINE_MULTIPLY_UP 6
#define FIELD_BASELINE_MULTIPLY_DOWN 7

/*
 * database baseline_calendar table
 * fields index
 */
#define FIELD_BASELINE_CALENDAR_ID 0
#define FIELD_BASELINE_CALENDAR_DATE 1
#define FIELD_BASELINE_CALENDAR_UTIL 2
#define FIELD_BASELINE_CALENDAR_SAMPLES 3
#define FIELD_BASELINE_CALENDAR_ALARM 4
#define FIELD_BASELINE_CALENDAR_MULTIPLY_UP 5
#define FIELD_BASELINE_CALENDAR_MULTIPLY_DOWN 6

/*
 * database service_metric table
 * fields index
 */
#define FIELD_SERVICE_METRIC_METRIC_ID 0
#define FIELD_SERVICE_METRIC_HOST_ID 1
#define FIELD_SERVICE_METRIC_SERVICE_ID 2
#define FIELD_SERVICE_METRIC_METRIC 3
#define FIELD_SERVICE_METRIC_UNIT 4
#define FIELD_SERVICE_METRIC_VALUE_MAX 5
#define FIELD_SERVICE_METRIC_VALUE_MIN 6
#define FIELD_SERVICE_METRIC_BASELINE_ID 7

/*
 * database service_perf table
 * fields index
 */
#define FIELD_SERVICE_PERF_ENTRY_TIME 0
#define FIELD_SERVICE_PERF_SERVICE_ID 1
#define FIELD_SERVICE_PERF_METRIC_ID 2
#define FIELD_SERVICE_PERF_PERF_VALUE 3
#define FIELD_SERVICE_PERF_WARNING 4
#define FIELD_SERVICE_PERF_CRITICAL 5
#define FIELD_SERVICE_PERF_BASELINE 6
#define FIELD_SERVICE_PERF_LOWER_LIMIT 7
#define FIELD_SERVICE_PERF_UPPER_LIMIT 8

/**
 * timestamp conversion macros
 */
#define SEC_DAY 86400
#define SEC_HOUR 3600
#define SEC_MIN 60

/**
 * MySQL related defines
 */
#define OP_OPSTATUS_FILE_ERRORS     "/usr/local/opmon/var/opstatus.errors"
#define OP_PERFDATA_FILE_ERRORS     "/usr/local/opmon/var/perfdata.errors"
#define OP_JOBSERVERS_FILE          "/usr/local/opmon/etc/jobservers"
#define DBOPCFG                     "opcfg"
#define DBOPPERF                    "opperf"
#define OP_LOG_MYSQL_ERROR          1

/**
 * function macros
 */
#define my_free(ptr) { if(ptr) { free(ptr); ptr = NULL; } }

struct service_metric {
	char	*entry_time;
	size_t	host_id;
	size_t	service_id;
	size_t	metric_id;
	size_t	baseline_id;
	char	*metric_name;
	char	*unit;
	char	*min;
	char	*max;
	char	*value;
	char	*warning;
	char	*critical;
	double	baseline;
	double	lower_limit;
	double	upper_limit;
};
typedef struct service_metric Service_Metric;

struct baseline {
	long		id;
	char		*name;
	char		*description;
	int			type;
	int			samples;
	int			alarm;
	double		multiply_up;
	double		multiply_down;
	char		*date;
	int			util;
};
typedef struct baseline Baseline;

struct snmp_check_parameters {
	char			*host_ip;
	int			 community_id;
	float			 warning;
	float			 critical;
	char			*warning_str;
	char			*critical_str;
	int			 warning_errors;
	int			 critical_errors;
	char			*filter;
	int			 timeout;
	int			 retries;
	char			*snmp_method;
	char			*snmp_method_oid;
	char			*snmp_method_ifname;
	unsigned long long	 ifspeed;
	unsigned long long	 ifspeed_upload;
	int			 ifindex;
	int			 bits;
	float			 tolerance;
	int 			adminstatus;
};

struct snmp_community {
	int	 community_id;
	char	*snmp_version;
	char	*community;
	char	*authentication_protocol;
	char	*authentication_passphrase;
	char	*security_level;
	char	*context_name;
	char	*security_username;
	char	*privacy_protocol;
	char	*privacy_protocol_passphrase;
	char	*destination_engine;
	int	 active;
};

struct snmp_leaf {
	unsigned long		 index;
	oid			*name;
	int			 name_length;
	char			*_char;
	long int 	 	 _integer;
	long unsigned		 _long_unsigned;
	long long unsigned	 _long_long_unsiged;
	struct snmp_leaf	*next;
	u_char			 type;
};

struct metric {
	char		*metric;
	char		*value;
	char		*unit;
	char		*warning;
	char		*critical;
	char		*value_min;
	char		*value_max;
	time_t		 current_time;
	struct metric	*next;
};
typedef struct metric Metric;

struct metric_node {
	int			 metric_id;
	char			*metric;
	char			*unit;
	double			 value;
	char			*value_min;
	char			*value_max;
	char			*warning;
	char			*critical;
	double			 hw_a;		// variable form he last HW to calculate de next
	double			 hw_t;		// variable form he last HW to calculate de next
	double			 hw;		// calculated holt-winters
	double			 hw_dev;	// calculated holt-winters
	double			 last_hw;	// calculated holt-winters
	double			 last_hw_dev;	// calculated holt-winters
	double			 upper_limit;
	double			 lower_limit;
	int			 baseline_broken;// 1 lower limit - 2 upper limit
	struct metric_node	*next;
};

/**
 * functions prototypes
 */
int op_write_to_memcache(char *memcache_key,void *tpl_buffer, int len, int expiration);
int op_read_from_memcache(char *memcache_key,char **tpl_buffer, size_t *dataSize);
void op_get_memcache_servers();
int init_community(struct snmp_community *snmp_parameters);
int init_snmp_check_params(struct snmp_check_parameters *params);
int prepare_snmp_session(struct snmp_session *s, struct snmp_check_parameters *params, struct snmp_community *snmp_parameters);
int get_community_by_id(int community_id, struct snmp_community *comm);
int do_query(char *query, char *db_host, char *db_user, char *db_pass, char *db_name, MYSQL_RES **res);
int parse_db_conf();
struct snmp_leaf *op_snmp_walk(oid *cpu_base_oid, size_t cpu_base_oid_len, struct snmp_session *session);
struct snmp_leaf *op_snmp_get(oid *base_oid, size_t base_oid_len, struct snmp_session *session);
struct snmp_leaf *get_leaf_from_vars(struct variable_list *vars);
int secs_to_date(long unsigned time, char **str_ret);
void copy_php_var(char *from, char *to);
void get_date(const char *format, char **date_str);
int get_baseline_calendar(Baseline **baseline);
void get_baseline_query(long baseline_id, char **query);
int get_baseline(long id, Baseline *baseline);
int get_metrics_by_host_and_service_id(long host_id, long service_id,
									Service_Metric **metrics);
int count_service_metrics(long host_id, long service_id);
double standard_deviation(double values[], int array_size);
int get_baseline_service_status(double hw_value,
								double std_dev,
								Baseline baseline,
								double value,
								const char* metric_name,
								char** msg);

int create_baseline_time_gaps(time_t check_timestamp,
								time_t check_interval_secs,
								const Baseline baseline,
								char **gaps);
int create_baseline_calendar_time_gaps(time_t check_timestamp,
									time_t check_interval_secs,
									const Baseline baseline,
									int ***calendar,
									char **gaps);
void timestamp_to_longdate(char *longdate, time_t timestamp);
int get_days_in_month(int year, int month);
int get_first_week_day_of_month(int year, int month);
int** gen_default_calendar(int samples, time_t timestamp);
int apply_calendar_baseline_exceptions(long baseline_id,
										int **calendar,
										int calendar_size,
										time_t timestamp);
int** gen_baseline_calendar(Baseline baseline, time_t timestamp);
long get_hostid_by_hostname(const char *hostname);
long get_serviceid_by_hostid_servicename(long host_id, const char *servicename);
long get_metricid_by_hostid_serviceid_metricname(long host_id,
												long service_id,
												const char *metricname);
void remove_invalid_char(char **orig);
long get_baselineid(long metric_id);
double holtwinters(double *perfdata, size_t size);
void winters(double *d, int seasons, int seasonlen);
double holts(double *demandv, int len);

extern char DBHOST[256];
extern char DBUSER[256];
extern char DBPASS[256];
extern char DBPORT[256];
extern int DEBUG;

#ifdef __cplusplus
};
#endif

#endif /* CHEADER_H_INCLUDED */
