/**
 * op_snmp_check.c - check speed and errors for the interfaces 
 */
#include <opservices.h>
#include <sys/types.h>
#include <pwd.h>


#ifdef DEBUG_MEMORY
#include <mcheck.h>
#endif

#define BUF_SIZE 512
#define STAT_DESCRIP_ERROR 99
#define BITSINBYTE 8
#define THIRTYTWO 4294967295ul
#define SIXTYFOUR 18446744073709551615ull
#define IFMIB_OIDS_TOTAL 9

struct memcache_stored {
	unsigned long long last_time;
	unsigned long long last_value[IFMIB_OIDS_TOTAL];
	unsigned long long service_state;
	char		*plugin_output;
	char		*perf_data;
};

int op_snmp_parse_args(int argc, char **argv, struct snmp_check_parameters *check_params, struct snmp_community *);
void get_interface_index(struct snmp_session *snmp, char **ifindex_tmp, int *ifs_found_in_walk, struct snmp_check_parameters *);
int get_ifname_by_method(struct snmp_session *sessp, int ifIndex , char **ifname, char *snmp_method_oid);
void set_opmon_state(int *service_state, int new_state);
int usage();

int main(int argc, char **argv) {
	void *sessp = NULL;
	struct snmp_session session;
	struct memcache_stored last_poll;
	struct snmp_pdu *pdu[IFMIB_OIDS_TOTAL];// = NULL;
	struct snmp_pdu *response[IFMIB_OIDS_TOTAL];// = NULL;
	oid anOID[IFMIB_OIDS_TOTAL][MAX_OID_LEN];
	size_t anOID_len[IFMIB_OIDS_TOTAL]; // = MAX_OID_LEN;
	struct variable_list *vars = NULL;
	unsigned long long result[IFMIB_OIDS_TOTAL];// = 0;
	unsigned long long insert_val[IFMIB_OIDS_TOTAL]; // = 0;
	double	kbps_val[IFMIB_OIDS_TOTAL], link_up_percentage,link_down_percentage,ifInDiscards_percentage, ifInErrors_percentage,ifOutDiscards_percentage,ifOutErrors_percentage;
	double download_converted, upload_converted;
	int status[IFMIB_OIDS_TOTAL], service_state; // = 0;
	char storedoid[IFMIB_OIDS_TOTAL][BUF_SIZE];
	char result_string[BUF_SIZE];
	int i, ret;
	char *snmp_invalid_value_char = NULL;
	char	*oid_tmp, *temp;
	int	poll_time, first_poll;
	time_t time_tmp;
	char	*ifname;
	char	*whatstheproblem;
	int	ifindex,	error_snmp, interfaces_differ;
	char	*ifindex_tmp;
	int has_to_update_ifindex=0;
	int out_of_bounds = 0, snmpv3_param_error = 0;
	char *snmpv3_param_error_tmp = NULL;
	char *perfdata_aux, *memcache_key;
	void		**tpl_mem_pack;
	size_t	tpl_pack_len;
	tpl_node *tn_metric;
	int ifs_found_in_walk, snmp_method_allow_walk, poll_values_differ;
	unsigned long long ifspeed_warning = 0;
	unsigned long long ifspeed_critical = 0;
	unsigned long long ifspeed_warning_upload = 0;
	unsigned long long ifspeed_critical_upload = 0;
	unsigned long long check_time;
	struct snmp_check_parameters *check_params = NULL;
	struct snmp_community *snmp_parameters = NULL;
	char *metric_unit_converted, *metric_unit_converted_ifspeed;
	unsigned long long ifspeed_converted, ifspeed_converted_upload;
	int  is_realtime = 0;
	struct passwd *passdata;
	float  tolerance = 1.5;

	/* Detects if plugin is being called by nagios or opmon realtime graph */
	passdata = getpwuid(geteuid());
	if ((passdata!=NULL) && strcmp(passdata->pw_name,"opuser")==0) {
		/* running from nagios / mod_gearman */
		is_realtime = 0;
	}
	else {
		/* running from realtime */
		is_realtime = 1;
	}

	char *op_oids[] = {
	                    ".1.3.6.1.2.1.2.2.1.10", // IF-MIB::ifInOctets    0
	                    ".1.3.6.1.2.1.2.2.1.13", // IF-MIB::ifInDiscards  1
	                    ".1.3.6.1.2.1.2.2.1.14", // IF-MIB::ifInErrors    2
	                    ".1.3.6.1.2.1.2.2.1.16", // IF-MIB::ifOutOctets   3
	                    ".1.3.6.1.2.1.2.2.1.19", // IF-MIB::ifOutDiscards 4
	                    ".1.3.6.1.2.1.2.2.1.20", // IF-MIB::ifOutErrors   5
	                    ".1.3.6.1.2.1.2.2.1.8",  // IF-MIB::ifOperStatus  6
	                    ".1.3.6.1.2.1.2.2.1.7",  // IF-MIB::ifAdminStatus 7
	                    "1.3.6.1.2.1.2.2.1.5"    // IF-MIB::ifSpeed       8
	};

	/* HCInOctets / HCOutOctets */
	char *op_oids64[] = {
	                     ".1.3.6.1.2.1.31.1.1.1.6", // IF-MIB::ifHCInOctets  0
	                     ".1.3.6.1.2.1.2.2.1.13",   // IF-MIB::ifInDiscards  1
	                     ".1.3.6.1.2.1.2.2.1.14",   // IF-MIB::ifInErrors    2
	                     ".1.3.6.1.2.1.31.1.1.1.10",// IF-MIB::ifHCOutOctets 3
	                     ".1.3.6.1.2.1.2.2.1.19",   // IF-MIB::ifOutDiscards 4
	                     ".1.3.6.1.2.1.2.2.1.20",   // IF-MIB::ifOutErrors   5
	                     ".1.3.6.1.2.1.2.2.1.8",    // IF-MIB::ifOperStatus  6
	                     ".1.3.6.1.2.1.2.2.1.7",    // IF-MIB::ifAdminStatus 7
	                     "1.3.6.1.2.1.2.2.1.5"      // IF-MIB::ifSpeed       8
	};

	/* Init things */
	last_poll.plugin_output=NULL;
	/* Check if counters have changed, because if they have not and elapsed time less than 30s, memcache is not update
	   and plugin output comes from memcache */
	poll_values_differ=0;
	for (i=0;i<IFMIB_OIDS_TOTAL;i++) {
		pdu[i]=NULL;
		response[i]=NULL;
		anOID_len[i]=MAX_OID_LEN;
		status[i]=0;
		result[i]=0;
		insert_val[i]=0;
		kbps_val[i]=0;
		last_poll.last_value[i]=0;
	}
	last_poll.last_time=0;
	last_poll.service_state=0;
	last_poll.plugin_output=NULL;
	last_poll.perf_data=NULL;

	op_get_memcache_servers();
	parse_db_conf();

	init_snmp("op-snmp-interface");

	check_params = (struct snmp_check_parameters *)malloc(sizeof(struct snmp_check_parameters));
	if (check_params  == NULL) {
		printf("Unable to allocate memory to parse command line parameters\n");
		return UNKNOWN;
	}

	snmp_parameters = (struct snmp_community *)malloc(sizeof(struct snmp_community));
	if (snmp_parameters  == NULL) {
		printf("Unable to allocate memory to parse command line parameters - snmp_parameters\n");
		return UNKNOWN;
	}

	/* Parse arguments, init snmp sessions and set parameters */	
	ret = op_snmp_parse_args(argc, argv, check_params, snmp_parameters);
	if (ret != OK) {
		return usage();
	}

	// Get the oid for the snmp_metrod chosen
	if (strcmp(check_params->snmp_method,"ifName")==0) {
		asprintf(&check_params->snmp_method_oid,"%s",".1.3.6.1.2.1.31.1.1.1.1");
	}
	else if (strcmp(check_params->snmp_method,"ifAlias")==0) {
		asprintf(&check_params->snmp_method_oid,"%s",".1.3.6.1.2.1.31.1.1.1.18");
	}
	else if (strcmp(check_params->snmp_method,"ifDescr")==0) {
		asprintf(&check_params->snmp_method_oid,"%s",".1.3.6.1.2.1.2.2.1.2");
	}
	else if (strcmp(check_params->snmp_method,"ifIndex")==0) {
		asprintf(&check_params->snmp_method_oid,"%s",".1.3.6.1.2.1.2.2.1.1");
	}
	else { // if nothing is passed, ifName it is
		asprintf(&check_params->snmp_method_oid,"%s",".1.3.6.1.2.1.31.1.1.1.1");
	}

	/* Convert thresholds from percentage to based on ifspeed */  
	if (check_params->ifspeed>0) {
		/* Download */
		ifspeed_warning=(check_params->ifspeed*check_params->warning)/100;
		ifspeed_critical=(check_params->ifspeed*check_params->critical)/100;

		/* Upload */
		ifspeed_warning_upload=(check_params->ifspeed_upload*check_params->warning)/100;
		ifspeed_critical_upload=(check_params->ifspeed_upload*check_params->critical)/100;
	} else {
		ifspeed_warning=0;
		ifspeed_critical=0;
	}

	if (DEBUG) {
		printf("Parameters\n");
		printf("\tbits %d\n", check_params->bits);
		printf("\tcommunity_id %d\n", check_params->community_id);
		printf("\thost_ip %s\n", check_params->host_ip);
		printf("\twarning %0.2f\n", check_params->warning);
		printf("\tcritical %0.2f\n", check_params->critical);
		printf("\twarning_errors %d\n", check_params->warning_errors);
		printf("\tcritical_errors %d\n", check_params->critical_errors);
		printf("\ttimeout %d\n", check_params->timeout);
		printf("\tretries %d\n", check_params->retries);
		printf("\tsnmp_method %s\n", check_params->snmp_method);
		printf("\tsnmp_method_ifname %s\n", check_params->snmp_method_ifname);
		printf("\tsnmp_method_oid %s\n", check_params->snmp_method_oid);
		printf("\tifspeed %lld\n", check_params->ifspeed);
		printf("\tifspeed_upload %lld\n", check_params->ifspeed_upload);
		printf("\tifindex %d\n", check_params->ifindex);
		printf("\ttolerance %0.2f\n", check_params->tolerance);
		printf("\tadminstatus %d\n\n", check_params->adminstatus);

		printf("\tsnmp_version %s\n", snmp_parameters->snmp_version);
		printf("\tcommunity %s\n", snmp_parameters->community);
		printf("\tauthentication_protocol %s\n", snmp_parameters->authentication_protocol);
		printf("\tauthentication_passphrase %s\n", snmp_parameters->authentication_passphrase);
		printf("\tsecurity_level %s\n", snmp_parameters->security_level);
		printf("\tcontext_name %s\n", snmp_parameters->context_name);
		printf("\tsecurity_username %s\n", snmp_parameters->security_username);
		printf("\tprivacy_protocol %s\n", snmp_parameters->privacy_protocol);
		printf("\tprivacy_protocol_passphrase %s\n", snmp_parameters->privacy_protocol_passphrase);
		printf("\tdestination_engine %s\n\n", snmp_parameters->destination_engine);
	}

	/* Creates memcache key - user realtime string to identify key running from opmon realtime graph */
	if (is_realtime == 1) {
		asprintf(&memcache_key, "%s_%s_realtime", check_params->host_ip, check_params->snmp_method_ifname);
	}
	else {
		asprintf(&memcache_key, "%s_%s", check_params->host_ip, check_params->snmp_method_ifname);
	}
	for (i=0;i<strlen(memcache_key);i++) {
		if ( ((int)memcache_key[i]<48) || ((int)memcache_key[i]>=58 && (int)memcache_key[i]<=64) || ((int)memcache_key[i]>=91 && (int)memcache_key[i]<=96) || ((int)memcache_key[i]>=123) ) {
			memcache_key[i]='_';
		}
	}
	
	if (DEBUG) {
		printf("memcache_key: %s\n", memcache_key);
	}

	/* Check time */
	time(&time_tmp);
	check_time=time_tmp;

	/* Tries to get lat pool from cache */
	if (op_read_from_memcache(memcache_key,(void *)&tpl_mem_pack, &tpl_pack_len) == OK ) {
		//last_collection, last_poll
		tn_metric = tpl_map("A(S(UUUUUUUUUUUss))", &last_poll);
		if (tpl_load(tn_metric, TPL_MEM | TPL_UFREE, tpl_mem_pack, tpl_pack_len)) {
			return ERROR;
		}
		tpl_unpack(tn_metric, 1);
		//tpl_free(tn_metric);
		poll_time=check_time - last_poll.last_time;
		if (DEBUG)
			printf("poll_time %d\n", poll_time);
		/* if cache older then 15 minutes, ignore it */
		if (poll_time > 960)
			first_poll=1;
		else if (poll_time <= 0)  /* clock has gonna backwards */
			first_poll=1;
		else
			first_poll=0;
		if (DEBUG)
			printf("Previus check in cache last_time %llu - poll_time: %d\n", last_poll.last_time, poll_time);
	}
	else {
		first_poll=1;
		//asprintf(&last_poll.plugin_output, "First poll");
		poll_time=0;
		last_poll.service_state=0;
		if (DEBUG)
			printf("First poll\n");
	}

	error_snmp=0;
	snmp_sess_init( &session );
	ret = prepare_snmp_session(&session, check_params, snmp_parameters);
	if (ret != OK) {
		printf("Unable to locate community id %d\n",check_params->community_id);
		return UNKNOWN;
	}

	sessp = snmp_sess_open(&session);
	if (!sessp) {
		printf("Could not open SNMP session %s\n", session.peername);
		exit(UNKNOWN);
	}

	ifname=NULL;
	interfaces_differ=0;
	snmp_method_allow_walk=0;
	ifindex=check_params->ifindex;

	if (!(strcmp(check_params->snmp_method,"ifIndex")==0)) {
		snmp_method_allow_walk=1;
		error_snmp=get_ifname_by_method(sessp, ifindex, &ifname, check_params->snmp_method_oid);
		if (DEBUG)
			printf("get_ifname_by_method %s - %s %d - ifname %s - error_snmp %d\n", check_params->snmp_method, session.peername, check_params->ifindex,ifname,error_snmp);
	}
	if ( (ifname!=NULL) && (check_params->snmp_method_ifname) ) {
		interfaces_differ=strcmp(ifname,check_params->snmp_method_ifname);
	}
	if ( snmp_method_allow_walk==1 && (interfaces_differ!=0 || error_snmp>0 || ifname==NULL) ) {
		ifindex_tmp=NULL;
		get_interface_index(sessp, &ifindex_tmp, &ifs_found_in_walk, check_params);
		if (DEBUG)
			printf("get_interface_index %s %d - ifindex_tmp %s - ifs_found_in_walk %d\n", session.peername, check_params->ifindex, ifindex_tmp, ifs_found_in_walk);
		/* if ifindex_tmp not null, it has founded the new OID for the interface */
		if (ifindex_tmp) {
			if (DEBUG)
				printf("SNMP info: New interface index for device: %s - ifname: %s - ifindex: %s - snmp_method %s", session.peername, check_params->snmp_method_ifname, ifindex_tmp,check_params->snmp_method);
			check_params->ifindex=atoi(ifindex_tmp);
			has_to_update_ifindex=1;
		}
		else {
			if (ifs_found_in_walk>0) { // snmp_walk worked but interface does not exist.
				// Interface does not exists or error retriving
				error_snmp=3;
				snmp_sess_close(sessp);
				sessp=NULL;
			}
		}
	}
	my_free(ifname);

	/* get last_collection from memcache, if exits, otherwise fist collection */
	// to be implemented

	if (DEBUG)
		printf("process snmp_pdu_create %d\n", check_params->ifindex);
	for (i=0;i<IFMIB_OIDS_TOTAL;i++) {
		pdu[i]=snmp_pdu_create(SNMP_MSG_GET);
		if (check_params->bits==32)
			asprintf(&oid_tmp,"%s.%d",op_oids[i],check_params->ifindex);
		else
			asprintf(&oid_tmp,"%s.%d",op_oids64[i],check_params->ifindex);
		read_objid(oid_tmp, anOID[i], &anOID_len[i]);
		sprintf(storedoid[i], " %s %d - %s %s %ld",check_params->snmp_method_ifname,check_params->ifindex,oid_tmp,session.community,session.version);
		my_free(oid_tmp);
		insert_val[i] = 0;
	}

	/* Add oids to snmplib */
	if (sessp!=NULL) {
		for (i=0;i<IFMIB_OIDS_TOTAL;i++) {
			snmp_add_null_var(pdu[i], anOID[i], anOID_len[i]);
		}
	}
	/* get the valies */
	if (sessp!=NULL) {
		for (i=0;i<IFMIB_OIDS_TOTAL;i++) {
			status[i] = snmp_sess_synch_response(sessp, pdu[i], &response[i]);
		}
	}
	else {
		for (i=0;i<IFMIB_OIDS_TOTAL;i++) {
			status[i] = STAT_DESCRIP_ERROR;
			snmp_free_pdu(pdu[i]);
		}
	}

	/* Collect response and process stats */
	for (i=0;i<IFMIB_OIDS_TOTAL;i++) {
		/* Only log errors for ifInOctets and ifOutOctets */
		if (error_snmp==3) {
			printf("SNMP Error: Interface %s does not exist on %s\n", check_params->snmp_method_ifname, session.peername);
			exit(UNKNOWN);
		}
		else if ( (status[i] == STAT_DESCRIP_ERROR) && ((i==0) || (i==3)) ) {
			if (snmpv3_param_error==0)
				printf("SNMP Error: (%s) Bad descriptor. %s\n", session.peername,(snmpv3_param_error_tmp==NULL)?"":snmpv3_param_error_tmp);
			else
				printf("SNMP Error: (%s) %s\n", session.peername,(snmpv3_param_error_tmp==NULL)?"":snmpv3_param_error_tmp);
			exit(UNKNOWN);
		}
		else if ( (status[i] == STAT_TIMEOUT) && ((i==0) || (i==3)) ) {
			printf("SNMP No response: (%s - %s)\n", session.peername, check_params->snmp_method_ifname);
			exit(UNKNOWN);
		}
		else if ( (status[i] != STAT_SUCCESS) && ((i==0) || (i==3)) ) {
			printf("SNMP Error: Unsuccessuful (%d) (%s - %s)\n", status[i], session.peername, check_params->snmp_method_ifname);
			exit(UNKNOWN);
		}
		else if ( (status[i] == STAT_SUCCESS && response[i]->errstat != SNMP_ERR_NOERROR) && ((i==0) || (i==3)) ) {
			printf("SNMP Error:  %ld/%s (%s - %s)", response[i]->errstat,snmp_errstring(response[i]->errstat), session.peername,check_params->snmp_method_ifname);
			exit(UNKNOWN);
		}
		/*		else if (status[i] == STAT_SUCCESS && response[i]->errstat == SNMP_ERR_NOERROR) {
				stats.polls++;
				}*/
	}

	/* Calculate tolerance for go over ifspeed */
	tolerance = (check_params->tolerance / 100) + 1;

	if (DEBUG) {
		printf("\nProcess SNMP results\n");
	}
	/* Gets results */
	// for (i=0;i<IFMIB_OIDS_TOTAL;i++) {
	for (i=IFMIB_OIDS_TOTAL-1;i>=0;i--) {
		if (DEBUG) {
			printf("Counter %d %s\n", i, op_oids[i]);
		}
		/* Liftoff, successful poll, process it */
		if (status[i] == STAT_SUCCESS && response[i]->errstat == SNMP_ERR_NOERROR) {
			vars = response[i]->variables;
#ifdef OLD_UCD_SNMP
			sprint_value(result_string, anOID[i], anOID_len[i], vars);
#else
			snprint_value(result_string, BUF_SIZE, anOID[i], anOID_len[i], vars);
#endif
			switch (vars->type) {
				/*
				 * Switch over vars->type and modify/assign result accordingly.
				 */
				case ASN_COUNTER64:
					if (DEBUG) printf("64-bit result: (%s@%s) %s\n", session.peername, storedoid[i], result_string);
					result[i] = vars->val.counter64->high;
					result[i] = result[i] << 32;
					result[i] = result[i] + vars->val.counter64->low;
					break;
				case ASN_COUNTER:
					if (DEBUG) printf("32-bit result: (%s@%s) %s\n", session.peername, storedoid[i], result_string);
					result[i] = (unsigned long) *(vars->val.integer);
					break;
				case ASN_INTEGER:
					if (DEBUG) printf("Integer result: (%s@%s) %s\n", session.peername, storedoid[i], result_string);
					result[i] = (unsigned long) *(vars->val.integer);
					break;
				case ASN_GAUGE:
					if (DEBUG) printf("32-bit gauge: (%s@%s) %s\n", session.peername, storedoid[i], result_string);
					result[i] = (unsigned long) *(vars->val.integer);
					break;
				case ASN_TIMETICKS:
					if (DEBUG) printf("Timeticks result: (%s@%s) %s\n", session.peername, storedoid[i], result_string);
					result[i] = (unsigned long) *(vars->val.integer);
					break;
				case ASN_OPAQUE:
					if (DEBUG) printf("Opaque result: (%s@%s) %s\n", session.peername, storedoid[i], result_string);
					result[i] = (unsigned long) *(vars->val.integer);
					break;
				default:
					if (i==0 || i==3) {
						if (DEBUG) printf("Unknown result type: (%s@%s) %s\n", session.peername, storedoid[i], result_string);
						status[i]=STAT_ERROR;
						if (error_snmp==0) {
							error_snmp=2;
							my_free(snmp_invalid_value_char);
							if (vars->type == 129) {
								printf("SNMP Error: Unknown result type: vars->type %u - problably your device does not support 64bits counters %s",vars->type, session.peername );
							}
							else {
								printf("SNMP Error: Unknown result type: vars->type %u - (%s@%s) %s",vars->type, session.peername, storedoid[i], result_string);
							}
							exit(UNKNOWN);
						}
					}
			}

			if (check_params->ifspeed<=0 && i == 8) {
				check_params->ifspeed = result[i];
				if (DEBUG) printf("Ifspeed collected: %lld\n", check_params->ifspeed);
			}
			if (check_params->ifspeed_upload<=0 && i == 8) {
				check_params->ifspeed_upload = result[i];
				if (DEBUG) printf("Ifspeed collected: %lld\n", check_params->ifspeed_upload);
			}
			if (DEBUG && i == 8) {
				printf("Tolerance multiplication: %0.2f\n\tifspeed %lld\n\tifspeed*tolerance %0.2f\n", tolerance, check_params->ifspeed, (check_params->ifspeed*tolerance));
			}

			/* Check if values differ */
			if ( ((i>=0) || (i<=5)) && first_poll==0 && last_poll.last_value[i]!=result[i] ) {
				poll_values_differ=1;
			}
			if (DEBUG) printf("poll_values_differ: %d\n", poll_values_differ);

			/* Gauge Type */
			if (check_params->bits == 0 || ((i==6) || (i==7)) ) {
				insert_val[i] = result[i];
			}
			else if ( (result[i] < last_poll.last_value[i]) && (first_poll==0) ) {
				if (check_params->bits == 32) {
					insert_val[i] = (THIRTYTWO - last_poll.last_value[i]) + result[i];
				}
				else {
					insert_val[i] = (SIXTYFOUR - last_poll.last_value[i]) + result[i];
				}
				if (DEBUG) {
					printf("*** Counter Wrap (%s@%s) [poll: %llu][last: %llu][insert: %llu]\n",	session.peername, storedoid[i], result[i], last_poll.last_value[i], insert_val[i]);
				} 
				/* Not a counter wrap and this is not the first poll */
			}
			else if ((last_poll.last_value[i] >= 0) && (first_poll==0)) {
				insert_val[i] = result[i] - last_poll.last_value[i];
				/* Print out SNMP result if verbose */
				if (DEBUG)
					printf("Calculate diff (%llu-%llu) = %llu\n", result[i], last_poll.last_value[i], insert_val[i]);
			}
			else {
				if (DEBUG)
					printf("First Poll, Normalizing %s %s\n", session.peername, check_params->snmp_method_ifname);
			}
			if (first_poll==0) {
				/* Get the real traffic in bits to compare to speed */
				/* Speed in kbps */
				if ( (i == 0) || (i == 3) ) { // In and Out Octets
					kbps_val[i]=(insert_val[i]/poll_time*8)/1000;
				}
				else {
					kbps_val[i] = insert_val[i];
				}
				if (DEBUG) {
					printf("kbps_val %0.2f - insert_val %llu\n", kbps_val[i], insert_val[i]);
				}
				/* If collected date is 50% bigger than ifSpeed, set it to ifspeed - if maxspeed 0, accept value */
				/* Download */
				if ( (i == 0) && (kbps_val[i] > (check_params->ifspeed*tolerance) || result[i] < 0) && (check_params->ifspeed!=0)) {
					if (DEBUG)
						printf("*** Out of Range (%s@%s) [kbps_val: %2.2f] [ifspeed: %llu]\n",session.peername, storedoid[i], kbps_val[i], check_params->ifspeed);
					//asprintf(&last_poll.plugin_output, "Out of Range kbps collected %2.2f - ifspeed: %llu", kbps_val[i], check_params->ifspeed);
					asprintf(&last_poll.plugin_output, "Interface speed over %2.2f%% %2.2f - ifspeed: %llu", check_params->tolerance, kbps_val[i], check_params->ifspeed);
					insert_val[i] = 0;
					first_poll=1;
					out_of_bounds = 1;
				}

				/* Upload */
				if ( (i == 3) && (kbps_val[i] > (check_params->ifspeed_upload*tolerance) || result[i] < 0) && (check_params->ifspeed_upload!=0)) {
					if (DEBUG)
						printf("*** Out of Range (%s@%s) [kbps_val: %2.2f] [ifspeed: %llu]\n",session.peername, storedoid[i], kbps_val[i], check_params->ifspeed_upload);
					//asprintf(&last_poll.plugin_output, "Out of Range kbps collected %2.2f - ifspeed: %llu", kbps_val[i], check_params->ifspeed);
					asprintf(&last_poll.plugin_output, "Interface speed over %2.2f%% %2.2f - ifspeed: %llu", check_params->tolerance, kbps_val[i], check_params->ifspeed_upload);
					insert_val[i] = 0;
					first_poll=1;
					out_of_bounds = 1;
				}
			}
		} /* STAT_SUCCESS */
		if (DEBUG) printf("\n");
	}

	/* Converte metric units the the small possible for bteter and fater interpretation */
	/* Download */
	ifspeed_converted = check_params->ifspeed;
	ifspeed_converted_upload = check_params->ifspeed_upload;
	/* Convert ifspeed  */
	i=0; // 0 kbps / 1 mbps / 2 gbps / 3 tbps
	while (ifspeed_converted>=1000 || ifspeed_converted_upload>=1000) {
	  	ifspeed_converted = ifspeed_converted / 1000;
	  	ifspeed_converted_upload = ifspeed_converted_upload / 1000;
		i++;
		if (i==3)
			break;
	}
	if (i==0)
		asprintf(&metric_unit_converted_ifspeed,"Kbps");
	else if (i==1)
		asprintf(&metric_unit_converted_ifspeed,"Mbps");
	else if (i==2)
		asprintf(&metric_unit_converted_ifspeed,"Gbps");
	else if (i==3)
		asprintf(&metric_unit_converted_ifspeed,"Tbps");
	else {
		ifspeed_converted=check_params->ifspeed;
		asprintf(&metric_unit_converted_ifspeed,"Kbps");
	}
	
	/* Convert upload and download speeds */
	download_converted=kbps_val[0];
	upload_converted=kbps_val[3];
	i=0; // 0 kbps / 1 mbps / 2 gbps / 3 tbps
	while (download_converted>=1000 || upload_converted>=1000) {
		i++;
		download_converted=download_converted/1000;
		upload_converted=upload_converted/1000;
		if (i==3)
			break;
	}
	if (i==0)
		asprintf(&metric_unit_converted,"Kbps");
	else if (i==1)
		asprintf(&metric_unit_converted,"Mbps");
	else if (i==2)
		asprintf(&metric_unit_converted,"Gbps");
	else if (i==3)
		asprintf(&metric_unit_converted,"Tbps");
	else {
		download_converted=kbps_val[0];
		upload_converted=kbps_val[3];
		asprintf(&metric_unit_converted,"Kbps");
	}

	service_state=0;
	/* If we have inoctets or outoctets */
	if ( status[0] == STAT_SUCCESS && response[0]->errstat == SNMP_ERR_NOERROR && status[3] == STAT_SUCCESS && response[3]->errstat == SNMP_ERR_NOERROR ) {
		if (result[6]<=1) { // ifOper UP
			if (poll_time>30 || poll_values_differ==1) {
				/* Check for bandwidth warning and critical */
				if (first_poll==0) {
					/* Check for discards or errors */
					/* Discards packets */
					ifInDiscards_percentage = kbps_val[1];
					ifOutDiscards_percentage = kbps_val[4];
					/* Errors packets */
					ifOutErrors_percentage = kbps_val[5];
					ifInErrors_percentage = kbps_val[2];
					if (DEBUG) {
						printf("\nPercentages\n");
						printf("ifInDiscards_percentage: %0.2f\n", ifInDiscards_percentage);
						printf("ifOutDiscards_percentage: %0.2f\n", ifOutDiscards_percentage);
						printf("ifOutErrors_percentage: %0.2f\n", ifOutErrors_percentage);
						printf("ifInErrors_percentage: %0.2f\n\n", ifInErrors_percentage);
					}
					temp=NULL;
					perfdata_aux = NULL;
					whatstheproblem = NULL;
					/* ifInDiscards */
					if (insert_val[1]>0) {
						/* if (ifInDiscards_percentage>check_params->critical_errors && check_params->critical_errors>0) {
							set_opmon_state(&service_state, 2);
						}
						else if (ifInDiscards_percentage>check_params->warning_errors && check_params->warning_errors>0) {
							set_opmon_state(&service_state, 1);
						} */
						asprintf(&perfdata_aux,"InDiscards %llu pkts", insert_val[1]);
						asprintf(&whatstheproblem, "InDiscards %llu pkts", insert_val[1]);
					}
					/* ifInErrors */
					if (insert_val[2]>0) {
						if (ifInErrors_percentage>check_params->critical_errors && check_params->critical_errors>0) {
							set_opmon_state(&service_state, 2);
						}
						else if (ifInErrors_percentage>check_params->warning_errors && check_params->warning_errors>0) {
							set_opmon_state(&service_state, 1);
						}
						if (perfdata_aux) {
							temp=perfdata_aux;
							asprintf(&perfdata_aux," %s / InErrors %llu", temp, insert_val[2]);
							my_free(temp);
						}
						else {
							asprintf(&perfdata_aux,"InErrors %llu", insert_val[2]);
						}
						if (whatstheproblem) {
							temp=whatstheproblem;
							asprintf(&whatstheproblem," %s / InErrors %llu", temp, insert_val[2]);
							my_free(temp);
						}
						else {
							asprintf(&whatstheproblem,"InErrors %llu", insert_val[2]);
						}
					}
					/* ifOutDiscards */
					if (insert_val[4]>0) {
						/* if (ifOutDiscards_percentage>check_params->critical_errors && check_params->critical_errors>0) {
							set_opmon_state(&service_state, 2);
						}
						else if (ifOutDiscards_percentage>check_params->warning_errors && check_params->warning_errors>0) {
							set_opmon_state(&service_state, 1);
						} */
						if (perfdata_aux) {
							temp=perfdata_aux;
							asprintf(&perfdata_aux," %s / OutDiscards %llu", temp, insert_val[4]);
							my_free(temp);
						}
						else {
							asprintf(&perfdata_aux,"OutDiscards %llu", insert_val[4]);
						}
						if (whatstheproblem) {
							temp=whatstheproblem;
							asprintf(&whatstheproblem," %s / OutDiscards %llu", temp, insert_val[4]);
							my_free(temp);
						}
						else {
							asprintf(&whatstheproblem,"OutDiscards %llu", insert_val[4]);
						}
					}
					/* ifOutErrors */
					if (insert_val[5]>0) {
						if (ifOutErrors_percentage>check_params->critical_errors && check_params->critical_errors>0) {
							set_opmon_state(&service_state, 2);
						}
						else if (ifOutErrors_percentage>check_params->warning_errors && check_params->warning_errors>0) {
							set_opmon_state(&service_state, 1);
						}
						if (perfdata_aux) {
							temp=perfdata_aux;
							asprintf(&perfdata_aux," %s / OutErrors %llu", temp, insert_val[5]);
							my_free(temp);
						}
						else {
							asprintf(&perfdata_aux,"OutErrors %llu", insert_val[5]);
						}
						if (whatstheproblem) {
							temp = whatstheproblem;
							asprintf(&whatstheproblem," %s / OutErrors %llu", temp, insert_val[5]);
							my_free(temp);
						}
						else {
							asprintf(&whatstheproblem,"OutErrors %llu", insert_val[5]);
						}
					}

					if (check_params->ifspeed>0) {
						/* Download and Upload rates */
						link_down_percentage = (100*kbps_val[0])/check_params->ifspeed;
						link_up_percentage = (100*kbps_val[3])/check_params->ifspeed_upload;
						if (DEBUG) {
							printf("\nPercentages\n");
							printf("link_down_percentage: %0.2f\n", link_down_percentage);
							printf("link_up_percentage: %0.2f\n", link_up_percentage);
						}
						if (check_params->critical>=check_params->warning) {
							if ((check_params->critical > 0) && ( (link_up_percentage > check_params->critical) || (link_down_percentage > check_params->critical) ) ) {
								set_opmon_state(&service_state, 2);
								if (whatstheproblem) {
									temp = whatstheproblem;
									asprintf(&whatstheproblem," %s / Download %.2f%s %.2f%% / Upload %.2f%s %.2f%% / Speed %lld%s/%lld%s", temp,download_converted,metric_unit_converted, link_down_percentage, upload_converted,metric_unit_converted,link_up_percentage, ifspeed_converted,metric_unit_converted_ifspeed, ifspeed_converted_upload, metric_unit_converted_ifspeed);
									my_free(temp);
								}
								else {
									asprintf(&whatstheproblem," Download %.2f%s %.2f%% / Upload %.2f%s %.2f%% / Speed %lld%s/%lld%s", download_converted,metric_unit_converted, link_down_percentage, upload_converted,metric_unit_converted,link_up_percentage, ifspeed_converted,metric_unit_converted_ifspeed, ifspeed_converted_upload, metric_unit_converted_ifspeed);
								}
							}
							else if ((check_params->warning>0) && ( (link_up_percentage > check_params->warning) || (link_down_percentage>check_params->warning) ) ) {
								set_opmon_state(&service_state, 1);
								if (whatstheproblem) {
									temp = whatstheproblem;
									asprintf(&whatstheproblem," %s / Download %.2f%s %.2f%% / Upload %.2f%s %.2f%% / Speed %lld%s/%lld%s", temp,download_converted,metric_unit_converted, link_down_percentage, upload_converted,metric_unit_converted,link_up_percentage, ifspeed_converted,metric_unit_converted_ifspeed, ifspeed_converted_upload, metric_unit_converted_ifspeed);
									my_free(temp);
								}
								else {
									asprintf(&whatstheproblem," Download %.2f%s %.2f%% / Upload %.2f%s %.2f%% / Speed %lld%s/%lld%s", download_converted,metric_unit_converted, link_down_percentage, upload_converted,metric_unit_converted,link_up_percentage, ifspeed_converted,metric_unit_converted_ifspeed, ifspeed_converted_upload, metric_unit_converted_ifspeed);
								}
							}
						}
						else {
							if ((check_params->critical>0) && ( (link_up_percentage < check_params->critical) || (link_down_percentage<check_params->critical) ) ) {
								set_opmon_state(&service_state, 2);
								if (whatstheproblem) {
									temp = whatstheproblem;
									asprintf(&whatstheproblem," %s / Download %.2f%s %.2f%% / Upload %.2f%s %.2f%% / Speed %lld%s/%lld%s", temp,download_converted,metric_unit_converted, link_down_percentage, upload_converted,metric_unit_converted,link_up_percentage, ifspeed_converted,metric_unit_converted_ifspeed, ifspeed_converted_upload, metric_unit_converted_ifspeed);
									my_free(temp);
								}
								else {
									asprintf(&whatstheproblem," Download %.2f%s %.2f%% / Upload %.2f%s %.2f%% / Speed %lld%s/%lld%s", download_converted,metric_unit_converted, link_down_percentage, upload_converted,metric_unit_converted,link_up_percentage, ifspeed_converted,metric_unit_converted_ifspeed, ifspeed_converted_upload, metric_unit_converted_ifspeed);
								}
							}
							else if ((check_params->warning>0) && ( (link_up_percentage<check_params->warning) || (link_down_percentage<check_params->warning) ) ) {
								set_opmon_state(&service_state, 1);
								if (whatstheproblem) {
									temp = whatstheproblem;
									asprintf(&whatstheproblem," %s / Download %.2f%s %.2f%% / Upload %.2f%s %.2f%% / Speed %lld%s/%lld%s", temp,download_converted,metric_unit_converted, link_down_percentage, upload_converted,metric_unit_converted,link_up_percentage, ifspeed_converted,metric_unit_converted_ifspeed, ifspeed_converted_upload, metric_unit_converted_ifspeed);
									my_free(temp);
								}
								else {
									asprintf(&whatstheproblem," Download %.2f%s %.2f%% / Upload %.2f%s %.2f%% / Speed %lld%s/%lld%s", download_converted,metric_unit_converted, link_down_percentage, upload_converted,metric_unit_converted,link_up_percentage, ifspeed_converted,metric_unit_converted_ifspeed, ifspeed_converted_upload, metric_unit_converted_ifspeed);
								}
							}							
						}
						// Download %.2f%s %.2f%% / Upload %.2%s %.2f%% / ifSpeed %lld%s
						// download_converted,metric_unit_converted,link_down_percentage,upload_converted,metric_unit_converted,link_up_percentage, ifspeed_converted, metric_unit_converted_ifspeed,
						if (whatstheproblem) {
								asprintf(&last_poll.plugin_output,"%s",whatstheproblem);
						}
						else { 
							asprintf(&last_poll.plugin_output, "Interface %s UP -  Download %.2f%s %.2f%% / Upload %.2f%s %.2f%% / Speed %lld%s/%lld%s %s", check_params->snmp_method_ifname, download_converted,metric_unit_converted, link_down_percentage, upload_converted,metric_unit_converted,link_up_percentage, ifspeed_converted,metric_unit_converted_ifspeed, ifspeed_converted_upload, metric_unit_converted_ifspeed, (perfdata_aux==NULL)?"":perfdata_aux);
						}
						asprintf(&last_poll.perf_data, "Download=%.2fkbps;%llu;%llu;0;%llu InDiscards=%llupkts;%d;%d;0; InErrors=%llupkts;%d;%d;0; Upload=%.2fkbps;%llu;%llu;0;%llu OutDiscards=%llupkts;%d;%d;0; OutErrors=%llupkts;%d;%d;0;",kbps_val[0],ifspeed_warning,ifspeed_critical,check_params->ifspeed,insert_val[1],check_params->warning_errors,check_params->critical_errors,insert_val[2],check_params->warning_errors,check_params->critical_errors,kbps_val[3],ifspeed_warning_upload,ifspeed_critical_upload,check_params->ifspeed_upload,insert_val[4],check_params->warning_errors,check_params->critical_errors,insert_val[5],check_params->warning_errors,check_params->critical_errors);
						my_free(perfdata_aux);
					}
					else {
						if (whatstheproblem) {
								asprintf(&last_poll.plugin_output,"%s",whatstheproblem);
						}
						else {
							asprintf(&last_poll.plugin_output, "Interface %s UP - Download %.2f%s / Upload %.2f%s / Speed %lld%s/%lld%s", check_params->snmp_method_ifname, download_converted,metric_unit_converted,upload_converted,metric_unit_converted,ifspeed_converted, metric_unit_converted_ifspeed, ifspeed_converted_upload, metric_unit_converted_ifspeed);
						}
						asprintf(&last_poll.perf_data, "Download=%.2fkbps;0;;; InDiscards=%llupkts;0;;; InErrors=%llupkts;0;;; Upload=%.2fkbps;0;;; OutDiscards=%llupkts;0;;; OutErrors=%llupkts;0;;;",kbps_val[0],insert_val[1],insert_val[2],kbps_val[3],insert_val[4],insert_val[5]);
						set_opmon_state(&service_state, 0);
					}
					printf("%s | %s\n", last_poll.plugin_output, last_poll.perf_data);
				}
				else {
					my_free(last_poll.plugin_output);
					if (out_of_bounds) {
						service_state = 3;
						asprintf(&last_poll.plugin_output,"Interface %s UP, but traffic flow is %2.2f%% over interface ifSpeed.", check_params->snmp_method_ifname, check_params->tolerance);
					} else {
						asprintf(&last_poll.plugin_output,"Interface %s UP", check_params->snmp_method_ifname);
					}

					if (last_poll.perf_data) {
						printf("%s | %s\n", last_poll.plugin_output, last_poll.perf_data);
					}
					else {
						printf("%s\n", last_poll.plugin_output);
					}
				}
			}
			else {
				if (last_poll.plugin_output==NULL)
					asprintf(&last_poll.plugin_output,"Interface %s UP", check_params->snmp_method_ifname);
				if (last_poll.perf_data) {
					printf("%s | %s\n", last_poll.plugin_output, last_poll.perf_data);
				}
				else {
					if (is_realtime == 1) {
						printf("%s | Download=0kbps;0;;; InDiscards=0pkts;0;;; InErrors=0pkts;0;;; Upload=0kbps;0;;; OutDiscards=0pkts;0;;; OutErrors=0pkts;0;;;\n", last_poll.plugin_output);
					}
					else {
						printf("%s\n", last_poll.plugin_output);
					}
				}
			}
		}
		else {
			if (result[7]==1) { // ifAdminStatus UP
				if (result[6]==2) { 
					printf("Interface %s Down\n", check_params->snmp_method_ifname);
				}
				else if (result[6]==3) {
					printf("Interface %s testing\n", check_params->snmp_method_ifname);
				}
				else if (result[6]==4) {
					printf("Interface %s unknown\n", check_params->snmp_method_ifname);
				}
				else if (result[6]==5) {
					printf("Interface %s dormant\n", check_params->snmp_method_ifname);
					exit(0);
				}
				else if (result[6]==6) {
					printf("Interface %s notPresent\n", check_params->snmp_method_ifname);
				}
				else if (result[6]==7) {
					printf("Interface %s lowerLayerDown\n", check_params->snmp_method_ifname);
				}
				else {
					printf("Interface %s Down\n", check_params->snmp_method_ifname);
				}
			}
			else {
				printf("Interface %s Administrative DOWN\n", check_params->snmp_method_ifname);
				exit(check_params->adminstatus);
			}
			exit(CRITICAL);
		}
	}
	/* Only if we received a positive result back do we update the last_value object */
	/* Check if ifInOctets and ifOutOctets checks were sucessfull or poll_time less than zero */
	if   ( (status[0] == STAT_SUCCESS && response[0]->errstat == SNMP_ERR_NOERROR) && 
			(status[3] == STAT_SUCCESS && response[3]->errstat == SNMP_ERR_NOERROR) ) {
		/* if poll_time greater than 30s or counter have changed, update memcache */
		if (poll_time>30 || poll_values_differ==1 || first_poll==1) {
			last_poll.last_time=check_time;
			last_poll.service_state=service_state;
			for (i=0;i<IFMIB_OIDS_TOTAL;i++) {
				last_poll.last_value[i] = result[i];
			}
			tn_metric = tpl_map("A(S(UUUUUUUUUUUss))", &last_poll);
			tpl_pack(tn_metric,1);
			tpl_dump(tn_metric, TPL_MEM, &tpl_mem_pack, &tpl_pack_len);
			op_write_to_memcache(memcache_key,tpl_mem_pack, tpl_pack_len, 900);
			tpl_free(tn_metric);
		}
	}

	if (sessp != NULL) {
		snmp_sess_close(sessp);
		for (i=0;i<IFMIB_OIDS_TOTAL;i++) {
			if (response[i] != NULL)
				snmp_free_pdu(response[i]);
		}
	}
	exit(service_state);
}

/* get interface index based on snmp method set on OpCFG */
void get_interface_index(struct snmp_session *snmp, char **ifindex_tmp, int *ifs_found_in_walk, struct snmp_check_parameters *check_params)
{
	int status, len;
	int i, found;
	int running = 1;
	struct snmp_pdu *pdu;
	struct snmp_pdu *response = NULL;
	struct variable_list *vars;
	size_t anOidLen = MAX_OID_LEN;
	oid anOid[MAX_OID_LEN];
	size_t rootLen = MAX_OID_LEN;
	oid root[MAX_OID_LEN];
	char *ret = NULL;
	char *str = NULL;
	char buf[1024];

	(*ifs_found_in_walk)=0;
	// Set SNMP request settings
	//if (snmp_parse_oid(".1.3.6.1.2.1.31.1.1.1.1", root, &rootLen) == NULL) {
	if (snmp_parse_oid(check_params->snmp_method_oid, root, &rootLen) == NULL) {
		//.1.3.6.1.2.1.2.2.1.1
		//snmp_perror(".1.3.6.1.2.1.31.1.1.1.1");
		//exit(1);
		return;
	}
	if (DEBUG) printf("get_interface_index - %s\n",check_params->snmp_method_oid);
	memmove(anOid, root, rootLen * sizeof(oid));
	anOidLen = rootLen;

	while(running) {
		pdu = snmp_pdu_create(SNMP_MSG_GETNEXT); 
		snmp_add_null_var(pdu, anOid, anOidLen);

		// Send request and get response
		if (DEBUG) printf("get_interface_index - while runnig %s\n", check_params->snmp_method_oid);
		status = snmp_sess_synch_response(snmp, pdu, &response);
		if (status == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR) {
			for ( vars = response->variables; vars; vars = vars->next_variable ) {
				if ((vars->name_length < rootLen) || (memcmp(root, vars->name, rootLen * sizeof(oid)) != 0)) {
					running = 0;
					continue;
				}
				if (vars->type == ASN_OCTET_STR) {
					str = (char *)malloc(1 + vars->val_len);
					memcpy(str, vars->val.string, vars->val_len);
					str[vars->val_len] = '\0';
					/* trim spaces from end of interface name */
					if (vars->val_len>0) {
						len=vars->val_len-1;
						while (len>0 && str[len]==' ') {
							str[len]=0;
							len--;
						}
						/* trim spaces from the begining of interface name */
						i=0;
						len=0;
						found=0;
						while (len<vars->val_len) {
							if (str[len]==' ' && found==0) {
								len++;
								continue;
							}
							found=1;
							str[i]=str[len];
							i++;
							len++;
						}
						str[i]=0;
					}
					memmove((char *) anOid, (char *) vars->name, vars->name_length * sizeof(oid));
					anOidLen = vars->name_length;
					(*ifs_found_in_walk)++;
					if (DEBUG) printf("get_interface_index - snmp_method_ifname <%s> - if_name <%s>\n", check_params->snmp_method_ifname, str);
					if (strcmp(check_params->snmp_method_ifname,str) == 0) {
						snprint_objid(buf, sizeof(anOid), anOid, anOidLen );
						ret = strtok( buf, "." );
						if (ret != NULL) {
							ret = strtok( NULL, "." );	
							*ifindex_tmp=strdup(ret);
							my_free(str);
							break;
						}
					}
					my_free(str);
				}
			}
		}
		else {
			running=0;
		}

		// Free resources
		if (response) {
			snmp_free_pdu(response);
			response=NULL;
		}

	}
	// Free resources
	if (response)
		snmp_free_pdu(response);
}

/* Gets ifName based on ifIndex */
int get_ifname_by_method(struct snmp_session *sessp, int ifIndex , char **ifname, char *snmp_method_oid)
{
	int status, len;
	int	ret, i, found;
	struct snmp_pdu *pdu;
	struct snmp_pdu *response=NULL;
	struct variable_list *vars;
	size_t anOID_len = MAX_OID_LEN;
	oid anOID[MAX_OID_LEN];

	char *oid_tmp=NULL;
	pdu=snmp_pdu_create(SNMP_MSG_GET);
	/* ifDescr and ifIndex composed */
	asprintf(&oid_tmp,"%s.%d",snmp_method_oid,ifIndex);
	read_objid(oid_tmp, anOID, &anOID_len);
	my_free(oid_tmp);
	snmp_add_null_var(pdu, anOID, anOID_len);
	status = snmp_sess_synch_response(sessp, pdu, &response);

	if (status == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR) {
		vars = response->variables;
		if (vars->type == ASN_OCTET_STR) {
			*ifname = (char *)malloc(1 + vars->val_len);
			memcpy(*ifname, vars->val.string, vars->val_len);
			*(*ifname+vars->val_len) = '\0';
			if (vars->val_len>0) {
				/* trim spaces from end of interface name */
				len=vars->val_len-1;
				while (len>0 && *(*ifname+len)==' ') {
					*(*ifname+len)=0;
					len--;
				}
				/* trim spaces from the begining of interface name */
				i=0;
				len=0;
				found=0;
				while (len<vars->val_len) {
					if (*(*ifname+len)==' ' && found==0) {
						len++;
						continue;
					}
					found=1;
					*(*ifname+i)=*(*ifname+len);
					i++;
					len++;
				}
				*(*ifname+i)=0;
			}
		}
		ret=response->errstat;
	}
	if (response) {
		ret=response->errstat;
	}
	else {
		ret=1;
	}
	// Free resources
	if (response)
		snmp_free_pdu(response);
	return(ret);
}

int op_snmp_parse_args(int argc, char **argv, struct snmp_check_parameters *params, struct snmp_community *snmp_parameters) {
	int arg;
	int ret;

	/* init defaults */
	init_community(snmp_parameters);

	/* init defaults */
	ret = init_snmp_check_params(params);
	if (ret != 0) {
		return ret;
	}
	
	while ( (arg = getopt(argc,argv,"v:c:a:A:l:n:u:x:X:Z:r:t:h:W:C:w:e:d:m:I:s:p:b:T:U:y:H:")) != EOF )  {
		
		if (DEBUG)
			printf("op_snmp_parse_args: %c - %s\n", arg,optarg);

		switch(arg) {

		case 'v':
			snmp_parameters->snmp_version = strdup(optarg);
			break;
		case 'c':
			snmp_parameters->community = strdup(optarg);
			break;
		case 'a':
			snmp_parameters->authentication_protocol = strdup(optarg);
			break;
		case 'A':
			snmp_parameters->authentication_passphrase = strdup(optarg);
			break;
		case 'l':
			snmp_parameters->security_level = strdup(optarg);
			break;
		case 'n':
			snmp_parameters->context_name = strdup(optarg);
			break;
		case 'u':
			snmp_parameters->security_username = strdup(optarg);
			break;
		case 'x':
			snmp_parameters->privacy_protocol = strdup(optarg);
			break;
		case 'X':
			snmp_parameters->privacy_protocol_passphrase = strdup(optarg);
			break;
		case 'Z':
			snmp_parameters->destination_engine = strdup(optarg);
			break;

		case 't':
			params->timeout = atoi(optarg);
			break;
		case 'r':
			params->retries = atoi(optarg);
			break;			
		case 'h':
			params->host_ip = strdup(optarg);
			break;
		case 'W':
			params->warning = atof(optarg);
			break;
		case 'C':
			params->critical = atof(optarg);
			break;
		case 'w':
			params->warning_errors = atof(optarg);
			break;
		case 'e':
			params->critical_errors = atof(optarg);
			break;
		case 'd':
			DEBUG = TRUE;
			break;
		case 'm':
			params->snmp_method = strdup(optarg);
			break;
		case 'I':
			params->snmp_method_ifname = strdup(optarg);
			break;
		case 's':
			params->ifspeed = atoi(optarg);
			break;
		case 'p':
			params->ifindex = atoi(optarg);
			break;
		case 'b':
			params->bits = atoi(optarg);
			break;			
		case 'T':
			params->tolerance = atof(optarg);
			break;
		case 'U':
			params->ifspeed_upload = atoi(optarg);
			break;
		case 'y':
			params->adminstatus = atoi(optarg);
			if (params->adminstatus<0 || params->adminstatus>3) {
				printf("Adminstatus return must by between 0 and 3\n");
				exit(3);
			}
			break;
		case 'H':
			usage();
			break;
		}
	}
	/* If no ifspeed for upload informed, used normal ifspeed for both */
	if (params->ifspeed_upload == 0)
		params->ifspeed_upload = params->ifspeed;

	if (params->host_ip == NULL || 
		snmp_parameters->snmp_version == NULL) // || parameter_must_exist<13)
		usage();

	return OK;
}

void set_opmon_state(int *service_state, int new_state) {
	if (new_state > *service_state) {
		*service_state = new_state;
	}
}

int usage() {
	printf("Usage:\n");
	printf("OPTIONS:\n");
	printf("  -h, --help			display this help message\n");
	printf("  -v 1|2c|3				specifies SNMP version to use\n");
	printf("  -V, --version			display package version number\n");
	printf("SNMP Version 1 or 2c specific\n");
	printf("  -c COMMUNITY			set the community string\n");
	printf("SNMP Version 3 specific\n");
	printf("  -a PROTOCOL			set authentication protocol (MD5|SHA)\n");
	printf("  -A PASSPHRASE			set authentication protocol pass phrase\n");
	printf("  -e ENGINE-ID			set security engine ID (e.g. 800000020109840301)\n");
	printf("  -E ENGINE-ID			set context engine ID (e.g. 800000020109840301)\n");
	printf("  -l LEVEL				set security level (noAuthNoPriv|authNoPriv|authPriv)\n");
	printf("  -n CONTEXT			set context name (e.g. bridge1)\n");
	printf("  -u USER-NAME			set security name (e.g. bert)\n");
	printf("  -x PROTOCOL			set privacy protocol (DES|AES)\n");
	printf("  -X PASSPHRASE			set privacy protocol pass phrase\n");
	printf("  -Z BOOTS,TIME			set destination engine boots/time\n");
	printf("General communication options\n");
	printf("  -r RETRIES			set the number of retries\n");
	printf("  -t TIMEOUT			set the request timeout (in seconds)\n");
	printf("  -h <host_ip:port>		set host IP and port\n");
	printf("  -W <warning_in_out>	set warning in and out in percentage\n");
	printf("  -C <critical_in_out>	set critical in and out in percentage\n");
	printf("  -w <warning_errors>	set warning error/discards in total packets from last poll\n");
	printf("  -e <critical_erros>	set critical error/discards in total packets from last poll\n");
	printf("  -m <snmp_method> 		ifName / ifDescr / ifAlias / ifIndex\n");
	printf("  -I <snmp_method_ifname>	interface name by selected method\n");
	printf("  -s <ifspeed>			interface speed - if no option passed, auto detect\n");
	printf("  -p <ifindex>			interface index\n");
	printf("  -b <counter bits>		32 / 64 - interface counters 32 ou 64 bits\n");
	printf("  -T <tolerance>		tolerance in percetage above ifSpeed\n");
	printf("  -U <ifspeed_upload>	ifSpeed for upload if diffrent from downstream\n");
	printf("  -y <admin status return>	return status if administrative down\n");
	printf("Debugging\n");
	printf("  -d			dump input/output packets in hexadecimal\n");

	exit(UNKNOWN);
}
