#include <opservices.h>

extern char 	*op_snmp_error;

int op_snmp_parse_args(int argc, char **argv, struct snmp_check_parameters *params) {

	int	arg;
	int	ret;

	ret = init_snmp_check_params(params);
	if (ret != OK)
		return ret;

	while( (arg = getopt(argc,argv,"i:h:w:c:t:r:d")) != EOF )  {

		switch(arg) {

		case 'i':
			params->community_id = atoi(optarg);
			break;

		case 'h':
			params->host_ip = strdup(optarg);
			memset(optarg, '\x0', strlen(optarg));
			break;

		case 'w':
			params->warning = atof(optarg);
			break;

		case 'c':
			params->critical = atof(optarg);
			break;

		case 't':
			params->timeout = atoi(optarg);
			break;

		case 'r':
			params->retries = atoi(optarg);
			break;

		case 'd':
			DEBUG = TRUE;
			break;

		}

	}

	if (params->community_id == 0 || params->host_ip == NULL)
		return ERROR;

	return OK;

}

int usage() {
	printf("Usage:\n");
	printf("op-snmp-load -h <host ip> -i <community id> [-w <warning>] [-c <critical>] [-t <snmp timeout>] [-r <retries>]\n");
	return UNKNOWN;
}

int main(int argc, char **argv) {

	struct snmp_session 		 session;
	float 				 cpu_usage_avg = 0;
	float 				 cpu_usage_sum = 0;
	char 				 plugin_output[MAX_PLUGIN_OUTPUT_LENGTH];
	int 				 status;
	int 				 cpus = 0;
	struct snmp_check_parameters 	*params = NULL;
	struct snmp_leaf 		*root_leaf = NULL;
	struct snmp_leaf 		*leaf = NULL;
	oid 				 cpu_base_oid[] = { 1, 3, 6, 1, 2, 1, 25, 3, 3, 1, 2 }; 

	op_get_memcache_servers();
	parse_db_conf();

	init_snmp("op-snmp-load-stats");

	params = (struct snmp_check_parameters *)malloc(sizeof(struct snmp_check_parameters));
	if (params == NULL) {
		printf("Unable to allocate memory to parse command line parameters\n");
		return UNKNOWN;
	}

	status = op_snmp_parse_args(argc,argv,params);
	if (status != OK) {
		return usage();
	}

	snmp_sess_init( &session );
	status = prepare_snmp_session( &session, params, NULL );
	if (status != OK) {
		printf("Unable to locate community id %d\n",params->community_id);
		return UNKNOWN;
	}

	root_leaf = op_snmp_walk(cpu_base_oid, sizeof(cpu_base_oid), &session);
	if (root_leaf == NULL && op_snmp_error != NULL) {
		printf("%s\n",op_snmp_error);
		return UNKNOWN;
	}

	for(leaf = root_leaf; leaf; leaf = leaf->next) {
		cpu_usage_sum += leaf->_integer;
		cpus++;
	}

	if (cpus == 0) {
		printf("Unable to locate cpu counters through snmp\n");
		SOCK_CLEANUP;
		return UNKNOWN;
	}

	cpu_usage_avg = cpu_usage_sum / cpus;

	snprintf(plugin_output, MAX_PLUGIN_OUTPUT_LENGTH - 1, "CPU Usage %.1f %% |cpu_usage=%.1f%%;%.2f;%.2f;0;100\n",
	    cpu_usage_avg,
	    cpu_usage_avg,
	    params->warning,
	    params->critical
	);

	status = OK;
	if (cpu_usage_avg > params->critical)
		status = CRITICAL;
	else if (cpu_usage_avg > params->warning)
		status = WARNING;

	printf("%s",plugin_output);
	free(params->host_ip);
	free(params);

	return status;

}
