#include <opservices.h>

extern char *op_snmp_error;

int op_snmp_parse_args(int argc, char **argv, struct snmp_check_parameters *params) {

	int arg;
	int ret;

	ret = init_snmp_check_params(params);
	if (ret != 0) {
		return ret;
	}

	while ( (arg = getopt(argc,argv,"i:h:f:t:r:d")) != EOF )  {

		switch(arg) {

		case 'i':
			params->community_id = atoi(optarg);
			break;

		case 'h':
			params->host_ip = strdup(optarg);
			memset(optarg, '\x0', strlen(optarg));
			break;

		case 't':
			params->timeout = atoi(optarg);
			break;

		case 'r':
			params->retries = atoi(optarg);
			break;

		case 'd':
			DEBUG = TRUE;
			break;

		case 'f':
			params->filter = strdup(optarg);
			memset(optarg, '\x0', strlen(optarg));
			break;

		}

	}

	if ( params->community_id == 0 || 
	     params->host_ip == NULL ||
	     params->filter == NULL ) {

		return ERROR;
	}

	return OK;


}

int usage() {
	printf("Usage:\n");
	printf("op-snmp-process-stats -h <host ip> -i <community id> -f <process name> [-t <snmp timeout>] [-r <snmp retries>]\n");
	return UNKNOWN;
}

int main(int argc, char **argv) {

	struct snmp_session session;
	char plugin_output[MAX_PLUGIN_OUTPUT_LENGTH];
	int status;
	struct snmp_check_parameters *params = NULL;
	struct snmp_leaf *root_leaf = NULL;
	struct snmp_leaf *leaf = NULL;
	int running_processes = 0;
	oid processes_names_base_oid[] = { 1, 3, 6, 1, 2, 1, 25, 4, 2, 1, 2 }; 

	op_get_memcache_servers();
	parse_db_conf();

	init_snmp("op-snmp-process-stats");

	params = (struct snmp_check_parameters *)malloc(sizeof(struct snmp_check_parameters));
	if (params == NULL) {
		printf("Unable to allocate memory to parse command line arguments\n");
		return UNKNOWN;
	}

	status = op_snmp_parse_args(argc,argv,params);
	if (status != OK) {
		return usage();
	}

	snmp_sess_init( &session );
	status = prepare_snmp_session( &session, params, NULL );
	if (status != OK) {
		printf("Unable to locate community id %d\n",params->community_id);
		return UNKNOWN;
	}

	root_leaf = op_snmp_walk(processes_names_base_oid, sizeof(processes_names_base_oid), &session);
	if (root_leaf == NULL) {
		if (op_snmp_error != NULL) {
			printf("%s\n",op_snmp_error);
		} else {
			printf("Unable to retrieve object oid through snmp\n");
		}
		return UNKNOWN;
	}

	for(leaf = root_leaf; leaf; leaf = leaf->next) {
		if (strcasecmp(leaf->_char,params->filter) == 0) {
			running_processes++;
		}
	}

	if (running_processes == 0) {
		printf("0 %s processes running |processes_running=0;;;;\n",params->filter);
		SOCK_CLEANUP;
		return CRITICAL;
	}

	snprintf(plugin_output, MAX_PLUGIN_OUTPUT_LENGTH - 1, "%d %s processes running |processes_running=%d;;;;\n", running_processes, params->filter, running_processes );
	printf("%s",plugin_output);
	return OK;

}
