#include <opservices.h>

extern char *op_snmp_error;

int op_snmp_parse_args(int argc, char **argv, struct snmp_check_parameters *params) {

	int arg;
	int ret;

	ret = init_snmp_check_params(params);
	if (ret != 0) {
		return ret;
	}

	params->critical = 300; // 5 minutes is the default
	while ( (arg = getopt(argc,argv,"i:h:c:t:r:d")) != EOF )  {

		switch(arg) {

		case 'i':
			params->community_id = atoi(optarg);
			break;

		case 'h':
			params->host_ip = strdup(optarg);
			memset(optarg, '\x0', strlen(optarg));
			break;

		case 'c':
			params->critical = atof(optarg) * 60;
			break;

		case 't':
			params->timeout = atoi(optarg);
			break;

		case 'r':
			params->retries = atoi(optarg);
			break;

		case 'd':
			DEBUG = TRUE;
			break;

		}

	}

	if (params->community_id == 0 || params->host_ip == NULL)
		return ERROR;

	return OK;


}

int usage() {
	printf("Usage:\n");
	printf("op-snmp-uptime -h <host ip> -i <community id> [-c <minutes>] [-t <snmp timeout>] [-r <retries>]\n");
	return UNKNOWN;
}

int main(int argc, char **argv) {

	struct snmp_session session;
	char *plugin_output = NULL;
	int status;
	struct snmp_check_parameters *params = NULL;
	struct snmp_leaf *root_leaf = NULL;
	oid cpu_base_oid[] = { 1, 3, 6, 1, 2, 1, 25, 1, 1, 0 }; 

	op_get_memcache_servers();
	parse_db_conf();

	init_snmp("op-snmp-uptime");

	params = (struct snmp_check_parameters *)malloc(sizeof(struct snmp_check_parameters));
	if (params == NULL) {
		printf("Unable to allocate memory to parse command line parameters\n");
		return UNKNOWN;
	}

	status = op_snmp_parse_args(argc,argv,params);
	if (status != OK) {
		return usage();
	}

	snmp_sess_init( &session );
	status = prepare_snmp_session( &session, params, NULL );
	if (status != OK) {
		printf("Unable to locate community id %d\n",params->community_id);
		return UNKNOWN;
	}

	root_leaf = op_snmp_get(cpu_base_oid, sizeof(cpu_base_oid), &session);
	if ((root_leaf == NULL) && (op_snmp_error != NULL)) {
		printf("%s\n",op_snmp_error);
		return UNKNOWN;
    } else if (root_leaf == NULL) {
        printf(
            "Could not open a SNMP session with the host address: %s\n",
            params->host_ip
        );
		return UNKNOWN;
    }

	if (root_leaf->type == ASN_INTEGER) {
		root_leaf->_long_unsigned = (long unsigned int)root_leaf->_integer / 100;
	} else {
		// convert to seconds
		root_leaf->_long_unsigned /= 100;
	}

	status = OK;
	if (root_leaf->_long_unsigned <= (long unsigned)params->critical)
		status = CRITICAL;

	secs_to_date(root_leaf->_long_unsigned, &plugin_output);

	printf("Uptime: %s",plugin_output);
	printf("| up=%d;;;;\n", (status == OK) ? 1 : 0);
	return status;

}
