#include <opservices.h>

extern char *op_snmp_error;

int op_snmp_parse_args(int argc, char **argv, struct snmp_check_parameters *params) {

	int arg;
	int ret;

	ret = init_snmp_check_params(params);
	if (ret != 0) {
		return ret;
	}

	while ( (arg = getopt(argc,argv,"i:h:f:t:r:d")) != EOF )  {

		switch(arg) {

		case 'i':
			params->community_id = atoi(optarg);
			break;

		case 'h':
			params->host_ip = strdup(optarg);
			memset(optarg, '\x0', strlen(optarg));
			break;

		case 'd':
			DEBUG = TRUE;
			break;

		case 't':
			params->timeout = atoi(optarg);
			break;

		case 'r':
			params->retries = atoi(optarg);
			break;

		case 'f':
			params->filter = strdup(optarg);
			memset(optarg, '\x0', strlen(optarg));
			break;

		}

	}

	if ( params->community_id == 0 || 
	     params->host_ip == NULL ||
	     params->filter == NULL ) {

		return ERROR;
	}

	return OK;


}

int usage() {
	printf("Usage:\n");
	printf("op-snmp-winservice -h <host ip> -i <community id> -f <service name> [-t <snmp timeout>] [-r <snmp retries>]\n");
	return UNKNOWN;
}

int main(int argc, char **argv) {

	struct snmp_session session;
	int status;
	int status_length = 0;
	struct snmp_check_parameters *params = NULL;
	struct snmp_leaf *root_leaf = NULL;
	struct snmp_leaf *leaf = NULL;
	struct snmp_leaf *status_leaf = NULL;
	oid services_names_base_oid[] = { 1, 3, 6, 1, 4, 1, 77, 1, 2, 3, 1, 1 }; 
	oid *status_oid = NULL;
	regex_t service_name_regex;
	char *regex_str;

	op_get_memcache_servers();
	parse_db_conf();

	init_snmp("op-snmp-winservice");

	params = (struct snmp_check_parameters *)malloc(sizeof(struct snmp_check_parameters));
	if (params == NULL) {
		printf("Unable to allocate memory to parse command line parameters\n");
		return UNKNOWN;
	}

	status = op_snmp_parse_args(argc,argv,params);
	if (status != OK) {
		return usage();
	}

	snmp_sess_init( &session );
	status = prepare_snmp_session( &session, params, NULL );
	if (status != OK) {
		printf("Unable to locate community id %d\n",params->community_id);
		return UNKNOWN;
	}

	root_leaf = op_snmp_walk(services_names_base_oid, sizeof(services_names_base_oid), &session);
	if (root_leaf == NULL) {
		if (op_snmp_error != NULL) {
			printf("%s\n",op_snmp_error);
		} else {
			printf("Unable to retrieve object oid through snmp\n");
		}
		return UNKNOWN;
	}


	for(leaf = root_leaf; leaf; leaf = leaf->next) {
		if (strcasecmp(leaf->_char,params->filter) == 0) {
			status_leaf = leaf;
			break;
		}
	}

	if (status_leaf == NULL) {

		/*try to locate service by regex*/
		asprintf(&regex_str,"^.*%s.*$", params->filter);
		if (regcomp(&service_name_regex,regex_str,REG_ICASE)) {
			printf("Invalid regex provided: %s\n",regex_str);
			SOCK_CLEANUP;
			return UNKNOWN;
		}

		for(leaf = root_leaf; leaf; leaf = leaf->next) {
			if (!regexec(&service_name_regex, leaf->_char, 0, NULL, 0)) {
				status_leaf = leaf;
				break;
			}
		}

		if (status_leaf == NULL) {
			printf("No services matching %s found\n",params->filter);
			SOCK_CLEANUP;
			return CRITICAL;
		}

	}

	status_oid = status_leaf->name;
	*(status_oid + 11) = 3;
	status_length = status_leaf->name_length * sizeof(oid);

	root_leaf = op_snmp_get(status_oid, status_length, &session);
	if (root_leaf == NULL) {
		if (op_snmp_error != NULL) {
			printf("%s\n",op_snmp_error);
		} else {
			printf("Unable to retrieve object oid through snmp\n");
		}
		return UNKNOWN;
	}

	
	if (root_leaf->_integer != 1) {
		status = CRITICAL;
		printf("%s service isn't running\n",status_leaf->_char);
	} else {
		status = OK;
		printf("%s service up and running\n",status_leaf->_char);
	}

	return status;

}
