#include <opservices.h>

char DBHOST[256];
char DBUSER[256];
char DBPASS[256];
char DBPORT[256];
int DEBUG = 0;
char 			snmp_community_serialization_string[] = "S(issssssssssi)";
static char 	*op_memcache_servers = NULL;
char	 		*op_snmp_error = NULL;

void copy_php_var(char *from, char *to) {

	int 	countquotes = 0;
	int	i = 0;
	int 	j = 0;

	for(i=0; i<strlen(from); i++) {

		if (countquotes == 1) {
			to[j] = from[i];
			j++;
		}

		if (from[i] == '"' || from[i] == '\'')
			countquotes++;
	}

	/* remove last quote */
	if (j > 0) {
		j--;
		to[j] = '\x0';
	}
}

/**
 * parse opmon's database configuration
 * file
 */
int parse_db_conf() {

	char 	*stringSearchDBHOST;
	char 	*stringSearchDBUSER;
	char 	*stringSearchDBPASS;
	char 	*stringSearchDBPORT;
	FILE 	*fp;
	char 	 line[256];

	/* retrieve the dbhost, username and password to connect to the mysql */
	fp = fopen(DBCONF, "r");
	if (fp == NULL)
		return ERROR;

	while (fgets(line, sizeof(line), fp)) {

		stringSearchDBHOST = strstr(line, "DBHOST");
		if (stringSearchDBHOST) {
			copy_php_var(line, DBHOST);
			continue;
		}

		stringSearchDBUSER = strstr(line, "DBUSER");
		if (stringSearchDBUSER) {
			copy_php_var(line, DBUSER);
			continue;
		}

		stringSearchDBPASS = strstr(line, "DBPASS");
		if (stringSearchDBPASS) {
			copy_php_var(line, DBPASS);
			continue;
		}

		stringSearchDBPORT = strstr(line, "DBPORT");
		if (stringSearchDBPORT) {
			copy_php_var(line, DBPORT);
		}
	}
	fclose(fp);

	if (__builtin_expect(DEBUG,FALSE)) {
		printf("dbhost: %s\n",DBHOST);
		printf("dbuser: %s\n",DBUSER);
		printf("dbpass: %s\n",DBPASS);
		printf("dbport: %s\n",DBPORT);
	}

	return OK;
}


/**
 * initiate a structure that contains
 * all valid command line parameters
 */
int init_snmp_check_params(struct snmp_check_parameters *params) {

	if (!params)
		return ERROR;

	params->filter = NULL;
	params->community_id = -1;
	params->bits = 64;
	params->host_ip = NULL;
	params->warning = 90;
	params->critical = 95;
	params->warning_errors = 5;
	params->critical_errors = 10;
	params->timeout = 2;
	params->retries = 1;
	params->snmp_method = NULL;
	params->snmp_method_oid = NULL;
	params->snmp_method_ifname = NULL;
	params->ifspeed = 0;
	params->ifspeed_upload = 0;
	params->ifindex = 1;
	params->tolerance = 50;
	params->adminstatus = 0;


	return OK;
}

int init_community(struct snmp_community *comm) {

	if (!comm)
		return ERROR;

	comm->community_id = -1;
	comm->snmp_version = NULL;
	comm->community = NULL;
	comm->authentication_protocol = NULL;
	comm->authentication_passphrase = NULL;
	comm->security_level = NULL;
	comm->context_name = NULL;
	comm->security_username = NULL;
	comm->privacy_protocol = NULL;
	comm->privacy_protocol_passphrase = NULL;
	comm->destination_engine = NULL;
	comm->active = 0;

	return OK;
}

/**
 * populates an snmp_sesion var with
 * community information.
 * community data is retrieved either
 * from memcache or database
 */
int prepare_snmp_session(struct snmp_session *s, struct snmp_check_parameters *params, struct snmp_community *snmp_parameters) {

	struct snmp_community 	 *comm;
	int 			 ret = 0;
	char 			*cp;
	char 			*endptr;

	if (snmp_parameters == NULL)  {
		comm = (struct snmp_community *)malloc(sizeof(struct snmp_community));

		if (init_community(comm) != OK)
			return ERROR;
		ret = get_community_by_id(params->community_id,comm);
		if (ret != OK)
			return ERROR;
	}
	else {
		comm = snmp_parameters;
	}

	s->peername = strdup(params->host_ip);

	if (!strcmp(comm->snmp_version, "1"))
		s->version = SNMP_VERSION_1;
	else if (!strcmp(comm->snmp_version, "3"))
		s->version = SNMP_VERSION_3;
	else if (!strcmp(comm->snmp_version,"2c"))
		s->version = SNMP_VERSION_2c;
	else
		return ERROR;

	netsnmp_ds_set_int(NETSNMP_DS_LIBRARY_ID,NETSNMP_DS_LIB_STRING_OUTPUT_FORMAT,NETSNMP_STRING_OUTPUT_ASCII);

	s->timeout = params->timeout * 1000000L;
	s->retries = params->retries;

	if (s->version == SNMP_VERSION_1 || s->version == SNMP_VERSION_2c) {
		s->community = (u_char *)strdup(comm->community);
		s->community_len = strlen(comm->community);

	} else {

		// SNMPV3 SPECIFIC VARIABLES
		if (comm->authentication_protocol) { // -a
			if (__builtin_expect(DEBUG,FALSE)) {
				printf("prepare_snmp_session: setting authentication_protocol\n");
			}
			if (!strcasecmp(comm->authentication_protocol, "MD5")) {
				s->securityAuthProto = usmHMACMD5AuthProtocol;
				s->securityAuthProtoLen = USM_AUTH_PROTO_MD5_LEN;
			} else if (!strcasecmp(comm->authentication_protocol,"SHA")) {
				s->securityAuthProto = usmHMACSHA1AuthProtocol;
				s->securityAuthProtoLen = USM_AUTH_PROTO_SHA_LEN;
			} else if (__builtin_expect(DEBUG,FALSE)) {
				printf("prepare_snmp_session: Invalid authentication protocol: %s",comm->authentication_protocol);
			}
		}

		if (comm->authentication_passphrase) { // -A
			if (__builtin_expect(DEBUG,FALSE)) {
				printf("prepare_snmp_session: setting authentication_passphrase\n");
			}
			s->securityAuthKeyLen = USM_AUTH_KU_LEN;
			if (generate_Ku( s->securityAuthProto,
			                 s->securityAuthProtoLen,
			                 (u_char *)comm->authentication_passphrase,
			                 strlen(comm->authentication_passphrase),
			                 s->securityAuthKey,
			                 &s->securityAuthKeyLen ) != SNMPERR_SUCCESS && DEBUG) {

				printf("prepare_snmp_session: invalid authentication passphrase\n");

			}

		}

		if (comm->security_level) { // -l

			if (__builtin_expect(DEBUG,FALSE)) {
				printf("prepare_snmp_session: setting security_level\n");
			}

			if (!strcasecmp(comm->security_level,"noAuthNoPriv")) {
				s->securityLevel = SNMP_SEC_LEVEL_NOAUTH;
			} else if (!strcasecmp(comm->security_level,"authNoPriv")) {
				s->securityLevel = SNMP_SEC_LEVEL_AUTHNOPRIV;
			} else if (!strcasecmp(comm->security_level,"authPriv")) {
				s->securityLevel = SNMP_SEC_LEVEL_AUTHPRIV;
			} else if (__builtin_expect(DEBUG,FALSE)) {
				printf("prepare_snmp_session: Invalid security level %s",comm->security_level);
			}
		}

		if (comm->context_name) { // -n
			if (__builtin_expect(DEBUG,FALSE)) {
				printf("prepare_snmp_session: setting context_name\n");
			}
			s->contextName = strdup(comm->context_name);
			s->contextNameLen = strlen(comm->context_name);
		}

		if (comm->security_username) { // -u
			if (__builtin_expect(DEBUG,FALSE)) {
				printf("prepare_snmp_session: setting security_username\n");
			}
			s->securityName = strdup(comm->security_username);
			s->securityNameLen = strlen(comm->security_username);
		}

		if (comm->privacy_protocol) { // -x
			if (__builtin_expect(DEBUG,FALSE)) {
				printf("prepare_snmp_session: setting privacy_protocol\n");
			}
			if (!strcasecmp(comm->privacy_protocol, "DES")) {
				s->securityPrivProto = usmDESPrivProtocol;
				s->securityPrivProtoLen = USM_PRIV_PROTO_DES_LEN;
			} else if (!strcasecmp(comm->privacy_protocol, "AES")) {
				s->securityPrivProto = usmAESPrivProtocol;
				s->securityPrivProtoLen = USM_PRIV_PROTO_AES_LEN;
			} else if (__builtin_expect(DEBUG,FALSE)) {
				printf("prepare_snmp_session: invalid privacy protocol\n");
			}
		}

		if (comm->privacy_protocol_passphrase) { // -X

			if (__builtin_expect(DEBUG,FALSE)) {
				printf("prepare_snmp_session: setting privacy_protocol_passphrase\n");
			}
			s->securityPrivKeyLen = USM_PRIV_KU_LEN;

			if (generate_Ku( s->securityAuthProto,
			                 s->securityAuthProtoLen,
			                 (u_char *)comm->privacy_protocol_passphrase,
			                 strlen(comm->privacy_protocol_passphrase),
			                 s->securityPrivKey,
			                 &s->securityPrivKeyLen)  != SNMPERR_SUCCESS && DEBUG) {

				printf("prepare_snmp_session: error processing privacy_protocol_passphrase\n");
			}
		}

		/**
 		 *  usually "engineBoots,engineTime"
		 *  we do not support old style parameter
		 *  as "engineBoots engineTime"
		 */
		if (comm->destination_engine) { // -Z
			if (__builtin_expect(DEBUG,FALSE)) {
				printf("prepare_snmp_session: setting destination_engine\n");
			}
			s->engineBoots = strtoul(comm->destination_engine, &cp, 10);
			if (cp != comm->destination_engine) {
				if (*cp == ',') {
					cp++;
					s->engineTime = strtoul(cp,&endptr,10);
				}
			}
		}

	}

	return OK;

}

/**
 * executes query on database.
 * returns either the number of affected
 * rows or ERROR
 */
int do_query(char *query, char *db_host, char *db_user, char *db_pass, char *db_name, MYSQL_RES **res) {

	MYSQL	*conn = NULL;
	int	 num_rows = 0;

	conn = mysql_init(NULL);
	if (!mysql_real_connect(conn,db_host,db_user,db_pass,db_name, 0, NULL, 0))
		return ERROR;

	if (mysql_query(conn, query) == 0) {
		*res = mysql_store_result(conn);
		num_rows = mysql_num_rows(*res);
		mysql_close(conn);
		return num_rows;
	}

	mysql_close(conn);
	return ERROR;

}

/**
 * get community either by memcache or
 * database
 */
int get_community_by_id(int community_id, struct snmp_community *comm) {

	tpl_node	*tn;
	char		*community_buffer = NULL;
	char		*community_memcache_index = NULL;
	char		*query = NULL;
	size_t		 buffer_size;
	int		 ret = 0;
	int		 num_fields = 0;
	int		 i;
	MYSQL_RES	*res = NULL;
	MYSQL_ROW	 row;

	asprintf(&community_memcache_index,"plugin_community_%d",community_id);

	ret = op_read_from_memcache(community_memcache_index, &community_buffer, &buffer_size);
	if (ret == OK) {
		// community retrieved from memcache,
		// we are ready to go.
		tn = tpl_map(snmp_community_serialization_string, comm);
		tpl_load(tn, TPL_MEM, community_buffer, buffer_size);
		tpl_unpack(tn,0);
		tpl_free(tn);
		free(community_memcache_index);
		free(community_buffer);
		return OK;
	}

	// get community from db and stores it on memcache
	asprintf(&query, "select * from snmp_communities where community_id='%d'", community_id);

	if (do_query(query,DBHOST,DBUSER,DBPASS,"opcfg",&res) <= 0)
		return ERROR;

	num_fields = mysql_num_fields(res);
	while((row = mysql_fetch_row(res))) {

		for (i=0; i<num_fields; i++) {

			if (row[i] == NULL || strlen(row[i]) == 0)
				continue;

			switch(i) {

			case FIELD_COMMUNITY_ID:
				comm->community_id = atoi(row[i]);
				break;

			case FIELD_SNMP_VERSION:
				comm->snmp_version = strdup(row[i]);
				break;

			case FIELD_COMMUNITY:
				comm->community = strdup(row[i]);
				break;

			case FIELD_AUTHENTICATION_PROTOCOL:
				comm->authentication_protocol = strdup(row[i]);
				break;

			case FIELD_AUTHENTICATION_PASSPHRASE:
				comm->authentication_passphrase = strdup(row[i]);
				break;

			case FIELD_SECURITY_LEVEL:
				comm->security_level = strdup(row[i]);
				break;

			case FIELD_CONTEXT_NAME:
				comm->context_name = strdup(row[i]);
				break;

			case FIELD_SECURITY_USERNAME:
				comm->security_username = strdup(row[i]);
				break;

			case FIELD_PRIVACY_PROTOCOL:
				comm->privacy_protocol = strdup(row[i]);
				break;

			case FIELD_PRIVACY_PROTOCOL_PASSPHRASE:
				comm->privacy_protocol_passphrase = strdup(row[i]);
				break;

			case FIELD_DESTINATION_ENGINE:
				comm->destination_engine = strdup(row[i]);
				break;

			case FIELD_ACTIVE:
				comm->active = atoi(row[i]);
				break;

			}

		}

	}

	// write community index on memcache
	tn = tpl_map(snmp_community_serialization_string,comm);
	tpl_pack(tn,0);
	tpl_dump(tn, TPL_MEM, &community_buffer, &buffer_size);
	tpl_free(tn);

	op_write_to_memcache(community_memcache_index, community_buffer, buffer_size, 60);

	mysql_free_result(res);
	free(community_memcache_index);

	return OK;

}


void op_get_memcache_servers() {

	FILE 		*fp;
	char 		*tmp = NULL;
	char		*line;
	size_t 		 len = 0;
	ssize_t		 read;
	int 		 lines = 0;

	op_memcache_servers = NULL;
	if (access(OP_JOBSERVERS_FILE, R_OK) == 0) {

		if ((fp=fopen(OP_JOBSERVERS_FILE,"r"))) {

			while((read = getline(&line, &len, fp)) != -1) {

				lines++;
				if (read > 0)
					line[read-1] = 0;

				if (op_memcache_servers == NULL) {
					asprintf(&op_memcache_servers, "--SERVER=%s", line);
				} else {
					asprintf(&tmp, "%s --SERVER=%s", op_memcache_servers, line);
					my_free(op_memcache_servers);
					op_memcache_servers = tmp;
					tmp = NULL;
				}
				my_free(line);
			}
			fclose(fp);
		}

	}

	if (!lines)
		asprintf(&op_memcache_servers, "--SERVER=127.0.0.1");

}

int op_read_from_memcache(char *memcache_key,char **tpl_buffer, size_t *dataSize) {

	char			*memcache_buffer;
	memcached_st		*memc;
	memcached_return	 rc;
	size_t			 mem_cache_length;
	uint32_t		 mem_cache_flags;

	if (op_memcache_servers == NULL)
		op_get_memcache_servers();

	if (__builtin_expect(DEBUG,FALSE)) {
		printf("memcache_get: memcache_key %s\n", memcache_key);
		printf("memcache_get: memcache_server %s\n", op_memcache_servers);
	}

	memc = memcached(op_memcache_servers, strlen(op_memcache_servers));
	memcache_buffer = memcached_get(memc, memcache_key, strlen(memcache_key), &mem_cache_length, &mem_cache_flags, &rc);
	memcached_free(memc);

	if (rc != MEMCACHED_SUCCESS) {
		if (__builtin_expect(DEBUG,FALSE))
			printf("memcache_get: not in memcache\n");
		return ERROR;
	}

	if (__builtin_expect(DEBUG,FALSE))
		printf("memcache_get: found in cache\n");

	*tpl_buffer = memcache_buffer;
	*dataSize = mem_cache_length;

	return OK;
}

int op_write_to_memcache(char *memcache_key,void *tpl_buffer, int len, int expiration) {

	memcached_st		*memc;
	memcached_return	 rc;

	if (op_memcache_servers == NULL)
		op_get_memcache_servers();

	memc = memcached(op_memcache_servers, strlen(op_memcache_servers));

	if (__builtin_expect(DEBUG,FALSE)) {
	       	printf("memcache_write: memcache_conf %s\n", op_memcache_servers);
	       	printf("memcache_write: memcache_key %s\n", memcache_key);
	}

	rc = memcached_set(memc, memcache_key, strlen(memcache_key), tpl_buffer, len, (time_t)expiration, (uint32_t)0);
	free(tpl_buffer);
	memcached_free(memc);

	if (rc != MEMCACHED_SUCCESS)
		return ERROR;

	return OK;
}

struct snmp_leaf *get_leaf_from_vars(struct variable_list *vars) {

	struct snmp_leaf	*leaf = NULL;
	size_t			 string_size;

	leaf = malloc(sizeof(struct snmp_leaf));
	if (leaf == NULL)
		return NULL;

	leaf->next = NULL;
	leaf->name = vars->name;
	leaf->name_length = vars->name_length;
	leaf->index = vars->name[vars->name_length - 1];
	leaf->type = vars->type;

	switch (vars->type) {

	case ASN_OCTET_STR:
		if (__builtin_expect(DEBUG,FALSE))
			printf("get_leaf_from_vars: query returned _char type\n");

		string_size = (vars->val_len * sizeof(char)) + sizeof(char);
		leaf->_char = malloc(string_size);
		if (leaf->_char == NULL)
			return NULL;

		memset(leaf->_char,0,string_size);
		strncpy(leaf->_char,(char *)vars->val.string,vars->val_len);
		break;

	case ASN_COUNTER64:
		if (__builtin_expect(DEBUG,FALSE))
			printf("get_leaf_from_vars: query returned _long_long_unsiged type\n");

		leaf->_long_long_unsiged = vars->val.counter64->high;
		leaf->_long_long_unsiged <<= 32;
		leaf->_long_long_unsiged |= vars->val.counter64->low;
		break;

	case ASN_TIMETICKS:
		if (__builtin_expect(DEBUG,FALSE))
			printf("get_leaf_from_vars: query returned _long_unsigned type\n");

		leaf->_long_unsigned = (long unsigned)*(vars->val.integer);
		break;

	case ASN_OPAQUE:
	case ASN_COUNTER:
	case ASN_INTEGER:
	case ASN_GAUGE:
		if (__builtin_expect(DEBUG,FALSE))
			printf("get_leaf_from_vars: query returned _integer type\n");

		leaf->_integer = (long int)*(vars->val.integer);
		break;

	default:
		break;
	}

	return leaf;

}


struct snmp_leaf *op_snmp_walk(oid *base_oid, size_t base_oid_len, struct snmp_session *session) {

	struct snmp_session	*sess_handle = NULL;
	oid			 root[MAX_OID_LEN];
	oid			 name[MAX_OID_LEN];
	size_t			 rootlen = 0;
	size_t			 namelen = 0;
	struct snmp_pdu		*pdu;
	struct snmp_pdu		*response;
	struct variable_list	*vars;
	int			 running = 0;
	int			 status = 0;
	struct snmp_leaf	*root_leaf = NULL;
	struct snmp_leaf	*current_leaf = NULL;
	struct snmp_leaf	*previous_leaf = NULL;

	SOCK_STARTUP;

	sess_handle = snmp_open(session);
	if (sess_handle == NULL) {
		snmp_sess_perror("op-snmp-load-stats",session);
		SOCK_CLEANUP;
		return NULL;
	}

	memmove(root, base_oid, base_oid_len);
	rootlen = base_oid_len / sizeof(oid);

	memmove(name, root, rootlen * sizeof(oid));
	namelen = rootlen;

	running = TRUE;
	while(running) {

		pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
		snmp_add_null_var(pdu, name, namelen);

		status = snmp_synch_response(sess_handle, pdu, &response);
		if (status == STAT_SUCCESS) {

			if (response->errstat != SNMP_ERR_NOERROR) {
				asprintf(&op_snmp_error,"Unexpected error. Unable to check host through snmp.");
				SOCK_CLEANUP;
				return NULL;
			}

			for (vars = response->variables; vars; vars = vars->next_variable) {

				if ((vars->name_length < rootlen) || (memcmp(root, vars->name, rootlen * sizeof(oid)) != 0)) {
					running = FALSE;
					continue;
				}

				if (__builtin_expect(DEBUG,FALSE))
					print_variable(vars->name, vars->name_length, vars);

				if (vars->type != SNMP_ENDOFMIBVIEW &&
				    vars->type != SNMP_NOSUCHOBJECT) {

					memmove( (char *) name, (char *)vars->name, vars->name_length * sizeof(oid) );
					namelen = vars->name_length;

					current_leaf = get_leaf_from_vars(vars);
					if (current_leaf == NULL)
						return NULL;

					if (root_leaf == NULL) {
						root_leaf = current_leaf;
					} else {
						previous_leaf->next = current_leaf;
					}
					previous_leaf = current_leaf;

				} else {
					running = FALSE;
					if (response->errstat != SNMP_ERR_NOSUCHNAME && DEBUG)
						printf("Error in packet. Reason: %s\n", snmp_errstring(response->errstat));
					continue;
				}

			}


		} else if (status == STAT_TIMEOUT) {
			asprintf(&op_snmp_error,"Timeout: No response from %s",session->peername);
			SOCK_CLEANUP;
			return NULL;

		} else {
			asprintf(&op_snmp_error,"Error connecting to remote host. Please verify your community configuration: %s",snmp_api_errstring(status));
			if (__builtin_expect(DEBUG,FALSE))
				snmp_sess_perror("op-snmp-load-stats",sess_handle);
			SOCK_CLEANUP;
			return NULL;
		}

	}

	snmp_free_pdu(response);
	snmp_close( sess_handle );

	return root_leaf;
}

struct snmp_leaf *op_snmp_get(oid *base_oid, size_t base_oid_len, struct snmp_session *session) {

	struct snmp_session	*sess_handle = NULL;
	struct snmp_pdu		*pdu;
	struct snmp_pdu		*response;
	struct variable_list	*vars;
	struct snmp_leaf	*root_leaf = NULL;
	int			 status = 0;

	SOCK_STARTUP;

	sess_handle = snmp_open(session);
	if (sess_handle == NULL) {
		snmp_sess_perror("op-snmp-load-stats",session);
		SOCK_CLEANUP;
		return NULL;
	}

	pdu = snmp_pdu_create(SNMP_MSG_GET);
	snmp_add_null_var(pdu, base_oid, base_oid_len/sizeof(oid));

	status = snmp_synch_response(sess_handle, pdu, &response);
	if (status == STAT_SUCCESS) {

		if (response->errstat != SNMP_ERR_NOERROR) {
			asprintf(&op_snmp_error,"Unexpected error. Unable to check host through snmp.");
			SOCK_CLEANUP;
			return NULL;
		}

		for (vars = response->variables; vars; vars = vars->next_variable) {

			if (__builtin_expect(DEBUG,FALSE))
				print_variable(vars->name, vars->name_length, vars);

			if (vars->type != SNMP_ENDOFMIBVIEW &&
			    vars->type != SNMP_NOSUCHOBJECT) {

				root_leaf = get_leaf_from_vars(vars);
				if (root_leaf == NULL)
					return NULL;

			} else {
				if (response->errstat != SNMP_ERR_NOSUCHNAME && DEBUG)
					printf("Error in packet. Reason: %s\n", snmp_errstring(response->errstat));
				goto out;
			}

		}


	} else if (status == STAT_TIMEOUT) {
		asprintf(&op_snmp_error,"Timeout: No response from %s",session->peername);
		SOCK_CLEANUP;
		return NULL;

	} else {
		asprintf(&op_snmp_error,"Error connecting to remote host. Please verify your community configuration: %s",snmp_api_errstring(status));
		if (__builtin_expect(DEBUG,FALSE))
			snmp_sess_perror("op-snmp-load-stats",sess_handle);

		SOCK_CLEANUP;
		return NULL;
	}


	out:
	snmp_free_pdu(response);
	snmp_close( sess_handle );

	return root_leaf;
}


int secs_to_date(long unsigned time, char **str_ret) {
	unsigned int	days = 0;
	unsigned int	hours = 0;
	unsigned int	mins = 0;
	unsigned int	seconds = 0;

	days = time / SEC_DAY;
	time %= SEC_DAY;

	hours = time / SEC_HOUR;
	time %= SEC_HOUR;

	mins = time / SEC_MIN;
	time %= SEC_MIN;

	seconds = time;

	asprintf(str_ret, "%d days %d hours %d minutes %d seconds", days,
	                                                            hours,
	                                                            mins,
	                                                            seconds
	);

	return OK;
}

int
get_season_idx(int idx, int seasonlen)
{
	return (int)idx / seasonlen;
}

int
get_period_idx(int idx, int seasonlen)
{
	return (int)idx % seasonlen;
}

int
calc_sum_by_season(double *demandv, int seasons, int seasonlen, double *ret)
{
	double	sum = 0;
	int	p = 0;
	int	i = 0;
	int	len = seasons * seasonlen;

	for(i=0; i<len; i++) {
		if (i != 0 && i % seasonlen == 0) {
			p = get_season_idx(i-1, seasonlen);
			ret[p] = sum;
			sum = 0;
		}
		sum += demandv[i];
	}
	p = get_season_idx(i-1, seasonlen);
	ret[p] = sum;
	return 0;
}

int
calc_pct_by_collect(double *demandv, int seasons, int seasonlen, double *totals, double *ret)
{
	int i;
	int p;
	int len = seasons * seasonlen;
	
	for(i=0; i<len; i++) {
		p = get_season_idx(i, seasonlen);
		ret[i] = demandv[i] / totals[p];
	}
	return 0;
}

void
get_ini_values(
    double *demandv, int seasons, int seasonlen, double *cs, double *as, double *bs)
{
	double	*totals;
	int	 len;
	int	 i;

	len = seasons * seasonlen;
	totals = calloc(seasons, sizeof(double));
	calc_sum_by_season(
	    demandv,
	    seasons,
	    seasonlen,
	    totals
	);

	calc_pct_by_collect(
	    demandv,
	    seasons, 
	    seasonlen,
	    totals,
	    cs
	);

	for(i=0; i<len; i++) {
		as[i] = demandv[i] / cs[i];
	}

	for(i=1; i<seasons; i++) {
		bs[i-1] = (totals[i] - totals[i-1]) / seasonlen;
	}

	free(totals);
}

void
winters(double *d, int seasons, int seasonlen)
{
	double	*c;
	double	*a;
	double	*b;
	double	*f;
	double	 alpha = 0.2;
	double	 beta = 0.3;
	double	 gamma = 0.25;
	int	 t;
	int	 fidx;
	int	 len = seasons * seasonlen;

	c = calloc(len * 2, sizeof(double));
	a = calloc(len * 2, sizeof(double));
	b = calloc(len * 2, sizeof(double));
	f = calloc(len * 2, sizeof(double));
	get_ini_values(
	    d,
	    seasons,
	    seasonlen,
	    c,
	    a,
	    b
	);

	for(t=0; t<=len+seasonlen; t++) {
		fidx = t+2;
		a[t + 1] = (alpha * a[t]) + ((1 - alpha) * (a[t] + b[t]));
		//a[t + 1] = (alpha * (d[t] / c[t])) + ((1 - alpha) * (a[t] + b[t]));
		//printf("(%.2f * (%.2f / %.2f)) + ((1 - %.2f) * (%.2f + %.2f)) = %.2f\n", alpha, d[t], c[t], alpha, a[t], b[t], a[t + 1]);

		b[t + 1] = (beta * (a[t+1] - a[t])) + ((1 - beta) * b[t]);
		//printf("(%.2f * (%.2f - %.2f)) + ((1 - %.2f) * %.2f) = %.2f\n", beta, a[t+1], a[t], beta, b[t], b[t+1]);

		c[t + seasonlen + 1] = (gamma * (d[t+1] / a[t+1])) + (1 - gamma) * c[t+1];
		//printf("(%.2f * (%.2f / %.2f)) + (1 - %.2f) * %.2f = %.4f\n", gamma, d[t+1] , a[t+1], gamma, c[t+1], c[t + seasonlen + 1]);

		f[fidx] = (a[t+1] + b[t+1]) * c[fidx];
		if (fidx >= len)
			printf("%.2f\n", f[fidx]);
	}

	free(c);
	free(a);
	free(b);
}




/**
 * holts
 *
 * Ft+1 = at + bt
 *
 * Ft+1 -> forecast
 * at   -> smoothed value(level) including the last data
 * bt   -> slope of the line given by bt
 */
double
holts(double *demandv, int len)
{
	double	alpha = 0.2;
	double	beta = 0.2;
	double	prevat = 0;
	double	prevbt = 0;
	double	at = 0;
	double	bt = 0;
	int	i = 0;

	if (len <= 0)
		return 0;

	prevbt = (demandv[1] - demandv[0]);
	prevat = demandv[0];
	for(i=1; i<len; i++) {
		at = (alpha * demandv[i]) + (1-alpha) * (prevat + prevbt);
		bt = beta * (at - prevat) + (1-beta) * prevbt;
		prevat = at;
		prevbt = bt;
	}

	return at + bt;
}


double
holtwinters(double *perfdata, size_t size)
{

	const double	delta_weight	= 0.2;
	const double	tendency_weight	= 0.2;
	const double	tendency_delta	= 1;
	size_t	index;
	double	tmp;
	double	next_value;
	double	next_delta;
	double	value;
	double	delta;
	double	tendency;
	double	*hw_points;
	double	ret;

	if (size <= 1) {
		return ERROR;
	}

	hw_points		= calloc(size, sizeof(double));
	value			= perfdata[1];
	delta			= perfdata[1] - perfdata[0];
	tendency		= value + (delta * tendency_delta);
	hw_points[0]	= tendency;

	for (index=1; index < size; index++) {
		tmp = tendency_weight * tendency;
		next_value = tmp + (1 - tendency_weight) * perfdata[index];
		tmp = delta_weight * delta;
		next_delta = tmp + (1 - delta_weight) * (next_value - value);

		hw_points[index] = next_value + (next_delta * tendency_weight);

		value		= next_value;
		delta		= next_delta;
		tendency	= hw_points[index];
	}

	ret = hw_points[size-1];
	free(hw_points);

	return ret;
}

double
standard_deviation(double values[], int array_size)
{
	int		i;
	double	mean, sum_values, variance;

	if (array_size <= 0) {
		return ERROR;
	}

	sum_values=0;
	for(i=0; i < array_size; i++) {
		sum_values+=values[i];
	}

	mean = sum_values / array_size;
	variance = 0.0;

	for(i=0; i < array_size; i++) {
		variance += pow(values[i] - mean, 2);
	}
	variance /= array_size;

	return sqrt(variance);
}

void
get_date(const char *format, char **date_str)
{
	time_t		timestamp = time(0);
	struct tm	*tmp_date;
	char		tmp_str[200];

	tmp_date = localtime(&timestamp);
	strftime(tmp_str, sizeof(tmp_str), format, tmp_date);
	asprintf(date_str, "%s", tmp_str);
}

int
get_baseline_service_status(double hw_value, double std_dev, Baseline baseline,
							double value, const char* metric_name, char** msg)
{
	double	up_limit;
	double	down_limit;

	up_limit	= hw_value + baseline.multiply_up * std_dev;
	down_limit	= hw_value - baseline.multiply_down * std_dev;
	(*msg)		= NULL;

	switch (baseline.alarm) {
		case BASELINE_ALARM_DISABLED:
			break;
		case BASELINE_ALARM_UP:

			if (value > up_limit) {
				asprintf(
					msg,
					"%s [%.2f > %.2f]",
					metric_name,
					value,
					up_limit
				);

				return CRITICAL;
			}

			break;
		case BASELINE_ALARM_DOWN:

			if (value < down_limit) {
				asprintf(
					msg,
					"%s [%.2f < %.2f]",
					metric_name,
					value,
					down_limit
				);

				return CRITICAL;
			}

			break;
		case BASELINE_ALARM_BOTH:

			if (value > up_limit) {
				asprintf(
					msg,
					"%s [%.2f > %.2f]",
					metric_name,
					value,
					up_limit
				);

				return CRITICAL;
			}

			if (value < down_limit) {
				asprintf(
					msg,
					"%s [%.2f < %.2f]",
					metric_name,
					value,
					down_limit
				);

				return CRITICAL;
			}

			break;
	}

	return OK;
}

/**
 *     This function gets a calendar configuration for a baseline and is called
 * by get_baseline function. If you want to call get_baseline_calendar directly,
 * you need to set "baseline->id" property before calling it. This function
 * won't save the name and description for the baseline, if you want it you
 * need to call get_baseline function. The baseline stored in baseline parameter
 * is ALWAYS for today.
 *
 * @param Baseline *baseline
 *     The variable where the baseline will be saved.
 * @return Int OK
 */
int
get_baseline_calendar(Baseline **baseline)
{
	MYSQL_RES	*res = NULL;
	MYSQL_ROW	row;
	char		*query, *date_str = NULL, *format;
	int			i, num_fields = 0;

	asprintf(&format, "%%Y-%%m-%%d");
	get_date(format, &date_str);

	asprintf(
		&query,
		"SELECT *				\
		FROM baseline_calendar	\
		WHERE baseline_id=%lu	\
		  AND date='%s'",
		(*baseline)->id,
		date_str
	);

	free(date_str);

	if (do_query(query, DBHOST, DBUSER, DBPASS, "opcfg", &res) <= 0) {
		free(query);
		return ERROR;
	}

	free(query);

	num_fields = mysql_num_fields(res);

	while ((row = mysql_fetch_row(res))) {
		for (i=0; i < num_fields; i++) {

			if ((row[i] == NULL) || (strlen(row[i]) == 0)) {
				continue;
			}

			switch(i) {
				case FIELD_BASELINE_CALENDAR_ID:
					continue;
				case FIELD_BASELINE_CALENDAR_DATE:
					asprintf(&(*baseline)->date, "%s", row[i]);
					break;
				case FIELD_BASELINE_CALENDAR_UTIL:
					(*baseline)->util = atoi(row[i]);
					break;
				case FIELD_BASELINE_CALENDAR_ALARM:
					(*baseline)->alarm = atoi(row[i]);
					break;
				case FIELD_BASELINE_CALENDAR_SAMPLES:
					(*baseline)->samples = atoi(row[i]);
					break;
				case FIELD_BASELINE_MULTIPLY_UP:
					(*baseline)->multiply_up = atof(row[i]);
					break;
				case FIELD_BASELINE_MULTIPLY_DOWN:
					(*baseline)->multiply_down = atof(row[i]);
					break;
			}

		}
	}

	mysql_free_result(res);

	return OK;
}

/**
 * This function build the query used to get a baseline from database.
 *
 * @param long baseline_id is a baseline id.
 * @param char **query is a string to store the query.
 */
void
get_baseline_query(long baseline_id, char **query)
{
	asprintf(
		query,
		"SELECT * FROM baseline	WHERE id=%lu",
		baseline_id
	);
}

/**
 *     This function get a baseline from cache if it's available in cache
 * otherwise from database.
 * @param Int id
 *     Is the id of the baseline that you want.
 * @param Baseline* baseline
 *     Is where the baseline properties will be saved.
 * @return Int
 *     OK or ERROR values. This values are defined in "opservces.h". This
 * function will return ERROR just when id parameter is invalid.
 */
int
get_baseline(long id, Baseline *baseline)
{
	char		*query						= NULL;
	MYSQL_RES	*res						= NULL;
	MYSQL_ROW	row;
	int			i, num_fields = 0;

	baseline->id			= 0;
	baseline->name			= NULL;
	baseline->description	= NULL;
	baseline->date			= NULL;
	baseline->type			= 0;
	baseline->samples		= 0;
	baseline->alarm			= 0;
	baseline->util			= -1; // not work day and not weekend, like undefined.
	baseline->multiply_up	= 0.0;
	baseline->multiply_down	= 0.0;

	if (id < 1) {
		return ERROR;
	}

	get_baseline_query(id, &query);

	parse_db_conf();

	if (do_query(query, DBHOST, DBUSER, DBPASS, "opcfg", &res) <= 0) {
		free(query);
		return ERROR;
	}

	free(query);
	num_fields = mysql_num_fields(res);

	while ((row = mysql_fetch_row(res))) {
		for (i=0; i < num_fields; i++) {

			if ((row[i] == NULL) || (strlen(row[i]) == 0)) {
				continue;
			}

			switch(i) {
				case FIELD_BASELINE_ID:
					baseline->id = atoi(row[i]);
					break;
				case FIELD_BASELINE_NAME:
					asprintf(&baseline->name, "%s", row[i]);
					break;
				case FIELD_BASELINE_DESCRIPTION:
					asprintf(&baseline->description, "%s", row[i]);
					break;
				case FIELD_BASELINE_TYPE:
					baseline->type = atoi(row[i]);
					break;
				case FIELD_BASELINE_SAMPLES:
					baseline->samples = atoi(row[i]);
					break;
				case FIELD_BASELINE_ALARM:
					baseline->alarm = atoi(row[i]);
					break;
				case FIELD_BASELINE_MULTIPLY_UP:
					baseline->multiply_up = atof(row[i]);
					break;
				case FIELD_BASELINE_MULTIPLY_DOWN:
					baseline->multiply_down = atof(row[i]);
					break;
			}

		}
	}



	if (baseline->type == BASELINE_TYPE_CALENDAR) {
		get_baseline_calendar(&baseline);
	}

	mysql_free_result(res);

	return OK;
}

/**
 *     This function returns all metrics from a service.
 * @param Long host_id
 * @param Long service_id
 * @param Service_Metric **metrics
 *     Is where the service metrics will be saved.
 * @return Int
 *     OK or ERROR values. This values are defined in "opservces.h".
 */
int
get_metrics_by_host_and_service_id(long host_id, long service_id, Service_Metric **metrics)
{
	tpl_node	*tn;
	char		*buffer = NULL;
	char		*memcache_index = NULL;
#ifdef __LP64__
	char		serialization_str[] = "A(S(sUUUUsssssssfff))";
#else
	char		serialization_str[] = "A(S(suuuusssssssfff))";
#endif

	MYSQL_RES	*res = NULL;
	MYSQL_ROW	row;
	char		*query;
	int			i, count, ret, num_fields = 0;
	size_t		buffer_size;
	Service_Metric	aux;

	asprintf(&memcache_index, "service_metric_%lu_%lu", host_id, service_id);

	ret = op_read_from_memcache(
		memcache_index,
		&buffer,
		&buffer_size
	);

	if (ret == OK) {
		// metrics retrieved from memcache,
		// we are ready to go.

		tn = tpl_map(serialization_str, &aux);
		tpl_load(tn, TPL_MEM, buffer, buffer_size);

		for(i=0; tpl_unpack(tn,1) > 0; i++) {
			memcpy(metrics[i], &aux, sizeof(Service_Metric));
		}

		tpl_free(tn);
		free(memcache_index);
		free(buffer);

		return OK;
	}

	if ((host_id < 1) || (service_id < 1)) {
		return ERROR;
	}

	asprintf(
		&query,
		"SELECT * "
		"FROM service_metric "
		"WHERE host_id=%lu"
		"	AND service_id=%lu",
		host_id,
		service_id
	);

	parse_db_conf();

	if (do_query(query, DBHOST, DBUSER, DBPASS, "opcfg", &res) <= 0) {
		free(query);
		return ERROR;
	}
	free(query);
	num_fields = mysql_num_fields(res);

	count = 0;
	while ((row = mysql_fetch_row(res))) {
		metrics[count]->baseline_id = 0;

		for (i=0; i < num_fields; i++) {

			if ((row[i] == NULL) || (strlen(row[i]) == 0)) {
				continue;
			}

			switch(i) {
				case FIELD_SERVICE_METRIC_METRIC_ID:
					metrics[count]->metric_id = atoi(row[i]);
					break;
				case FIELD_SERVICE_METRIC_HOST_ID:
					metrics[count]->host_id = atoi(row[i]);
					break;
				case FIELD_SERVICE_METRIC_SERVICE_ID:
					metrics[count]->service_id = atoi(row[i]);
					break;
				case FIELD_SERVICE_METRIC_METRIC:
					metrics[count]->metric_name = strdup(row[i]);
					break;
				case FIELD_SERVICE_METRIC_UNIT:
					metrics[count]->unit = strdup(row[i]);
					break;
				case FIELD_SERVICE_METRIC_VALUE_MAX:
					metrics[count]->max = strdup(row[i]);
					break;
				case FIELD_SERVICE_METRIC_VALUE_MIN:
					metrics[count]->min = strdup(row[i]);
					break;
				case FIELD_SERVICE_METRIC_BASELINE_ID:
					metrics[count]->baseline_id = atoi(row[i]);
					break;
			}
		}
		count++;
	}

	// write metrics on memcache
	tn = tpl_map(serialization_str, &aux);
	for(i=0; i < count; i++) {
		aux = *metrics[i];
		tpl_pack(tn,1);
	}
	tpl_dump(tn, TPL_MEM, &buffer, &buffer_size);

	op_write_to_memcache(
		memcache_index,
		buffer,
		buffer_size,
		300
	);

	tpl_free(tn);
	free(memcache_index);
	mysql_free_result(res);

	return OK;
}

/**
 *     This function returns the number of service's metrics.
 * @param Long host_id
 * @param Long service_id
 * @return Int
 *     The number of service's metrics.
 */
int
count_service_metrics(long host_id, long service_id)
{
	MYSQL_RES	*res				= NULL;
	char		*query				= NULL;
	int			number_of_rows		= 0;
	char		*str_aux			= NULL;
	int 		ret;
	tpl_node	*tn;
	char		*buffer				= NULL;
	char		*memcache_index		= NULL;
	char		serialization_str[]	= "s";
	size_t		buffer_size;

	asprintf(&memcache_index, "service_metric_count_%lu_%lu", host_id, service_id);

	ret = op_read_from_memcache(
		memcache_index,
		&buffer,
		&buffer_size
	);

	if (ret == OK) {
		// number of rows retrieved from memcache,
		// we are ready to go.
		tn = tpl_map(serialization_str, &str_aux);
		tpl_load(tn, TPL_MEM, buffer, buffer_size);
		tpl_unpack(tn,0);
		number_of_rows = atoi(str_aux);

		tpl_free(tn);
		free(memcache_index);
		free(buffer);
		free(str_aux);

		return number_of_rows;
	}

	if ((host_id < 1) || (service_id < 1)) {
		return ERROR;
	}

	asprintf(
		&query,
		"SELECT * "
		"FROM service_metric "
		"WHERE host_id=%lu "
		"	AND service_id=%lu",
		host_id,
		service_id
	);

	parse_db_conf();

	number_of_rows = do_query(query, DBHOST, DBUSER, DBPASS, "opcfg", &res);
	if (number_of_rows <= 0) {
		mysql_free_result(res);
		free(query);

		return ERROR;
	}


	asprintf(&str_aux, "%d", number_of_rows);

	// write number of rows on memcache
	tn = tpl_map(serialization_str, &str_aux);
	tpl_pack(tn,0);
	tpl_dump(tn, TPL_MEM, &buffer, &buffer_size);

	op_write_to_memcache(
		memcache_index,
		buffer,
		buffer_size,
		300
	);

	tpl_free(tn);
	mysql_free_result(res);
	free(str_aux);
	free(query);
	free(memcache_index);

	return number_of_rows;
}

/**
 * Create data gaps to HW processing
 */
int
create_baseline_time_gaps(time_t check_timestamp, time_t check_interval_secs,
						const Baseline baseline, char **gaps)
{

	size_t	i;
	size_t	seasonality = 0;
	time_t	tmp_time;
	char	date1[32];
	char	date2[32];
	struct	tm *check_date;
	struct	tm *gap_date;
	int		last_isdst;

	check_date = localtime(&check_timestamp);
	last_isdst = check_date->tm_isdst;

	check_interval_secs /= 2;

	switch(baseline.type) {
		case BASELINE_TYPE_DAY:			seasonality = 86400;	break;
		case BASELINE_TYPE_WEEK:		seasonality = 604800;	break;
		case BASELINE_TYPE_MONTH:		seasonality = 2592000;	break;
		case BASELINE_TYPE_CALENDAR:	return ERROR;			break;
	}

	tmp_time = check_timestamp;
	for (i=baseline.samples; i > 0; i--) {
		tmp_time -= seasonality;

		/**
		 * This source code block is need to adjust timestamp because of summertime.
		 */
		gap_date = localtime(&tmp_time);
		if (gap_date->tm_isdst != last_isdst) {
			if ((gap_date->tm_isdst > 0) && (last_isdst == 0)) {
				tmp_time -= 3600;
			} else if ((gap_date->tm_isdst == 0) && (last_isdst > 0)) {
				tmp_time += 3600;
			}

			last_isdst = gap_date->tm_isdst;
		}

		timestamp_to_longdate(date1, (tmp_time - check_interval_secs));
		timestamp_to_longdate(date2, (tmp_time + check_interval_secs));

		if (*gaps) {
			asprintf(
				gaps,
				"%s or (entry_time > '%s' and entry_time < '%s')",
				*gaps,
				date1,
				date2
			);
		} else {
			asprintf(
				gaps,
				" (entry_time > '%s' and entry_time < '%s')",
				date1,
				date2
			);
		}
	}

	return OK;
}

int
create_baseline_calendar_time_gaps(time_t check_timestamp,
						time_t check_interval_secs,
						const Baseline baseline,
						int ***calendar,
						char **gaps)
{

	size_t	i, j;
	size_t	day_secs = 86400;
	time_t	tmp_time;
	char	date1[32];
	char	date2[32];
	char	date[20];
	int		today_status;
	int		cardinal_position_today = 0;
	int		cardinal_position;
	int		year, month, day, sample_day;
	int		found_samples = 0;
	int		calendar_size = baseline.samples * 2;
	int		month_days;
	char	*str_aux = NULL;
	struct	tm *check_date;
	struct	tm *gap_date;
	int		last_isdst;

	check_date = localtime(&check_timestamp);
	last_isdst = check_date->tm_isdst;

	check_interval_secs /= 2;

	if (baseline.type != BASELINE_TYPE_CALENDAR) {
		return ERROR;
	}

	timestamp_to_longdate(date, check_timestamp);

	year	= atoi(strtok(date, "-"));
	month	= atoi(strtok(NULL, "-"));
	day		= atoi(strtok(NULL, "-"));
	today_status = (*calendar)[0][day-1];

	if (today_status < 0) {
		return ERROR;
	}

	for (i=0; i < day; i++) {
		if ((*calendar)[0][i] == today_status) {
			cardinal_position_today++;
		}
	}

	tmp_time = check_timestamp - (day * day_secs);

	for (i=1; i < calendar_size; i++) {
		cardinal_position = 0;
		month_days = 0;
		sample_day = -1;

		for (j=0; j < 31; j++) {
			if ((*calendar)[i][j] == today_status) {
				cardinal_position++;
			}

			if (cardinal_position == cardinal_position_today) {
				sample_day = j + 1;
			}

			// valid day for this month
			if ((*calendar)[i][j] >= 0) {
				month_days++;
			} else {
				break;
			}
		}

		if (sample_day == -1) {
			tmp_time -= (month_days * day_secs);
			continue;
		}

		found_samples++;
		tmp_time -= ((month_days - sample_day) * day_secs);

		/**
		 * This source code block is need to adjust timestamp because of summertime.
		 */
		gap_date = localtime(&tmp_time);
		if (gap_date->tm_isdst != last_isdst) {
			if ((gap_date->tm_isdst > 0) && (last_isdst == 0)) {
				tmp_time -= 3600;
			} else if ((gap_date->tm_isdst == 0) && (last_isdst > 0)) {
				tmp_time += 3600;
			}

			last_isdst = gap_date->tm_isdst;
		}

		timestamp_to_longdate(date1, (tmp_time - check_interval_secs));
		timestamp_to_longdate(date2, (tmp_time + check_interval_secs));

		if (*gaps) {
			asprintf(
				&str_aux,
				"%s or (entry_time > '%s' and entry_time < '%s')",
				*gaps,
				date1,
				date2
			);

			free((*gaps));
			asprintf(gaps, "%s", str_aux);
			free(str_aux);
		} else {
			asprintf(
				gaps,
				" (entry_time > '%s' and entry_time < '%s')",
				date1,
				date2
			);
		}

		if (found_samples == baseline.samples) {
			break;
		}

		tmp_time -= (sample_day * day_secs);
		timestamp_to_longdate(date1, (tmp_time - check_interval_secs));
	}

	return OK;
}

/*
 * convert timestamp to string in format YYYY-MM-DD HH:MM:SS
 */
void
timestamp_to_longdate(char *longdate, time_t timestamp)
{
	struct tm *system_date;

	system_date = localtime(&timestamp);
	sprintf(
		longdate,
		"%d-%02d-%02d %02d:%02d:%02d",
		system_date->tm_year+1900,
		system_date->tm_mon+1,
		system_date->tm_mday,
		system_date->tm_hour,
		system_date->tm_min,
		system_date->tm_sec
	);
}

int
get_days_in_month(int year, int month)
{
	if ((year < 0) || (month < 0)) {
		return ERROR;
	}

	struct tm system_date = {0};

	system_date.tm_year	= year;
	system_date.tm_mon	= month + 1;

	mktime(&system_date);

	return system_date.tm_mday;
}

int
get_first_week_day_of_month(int year, int month)
{
	struct tm	*system_date;
	time_t		timestamp;

	time(&timestamp);
	system_date = localtime(&timestamp);

	system_date->tm_year	= year;
	system_date->tm_mon		= month;
	system_date->tm_mday	= 1;
	system_date->tm_hour	= 1;
	system_date->tm_min		= 0;
	system_date->tm_sec		= 0;

	timestamp = mktime(system_date);
	system_date = localtime(&timestamp);

	return system_date->tm_wday;
}

int**
gen_default_calendar(int calendar_size, time_t timestamp)
{
	struct tm	*system_date;
	int			year, first_month_wday;
	int			i = 0;
	int			j = 0;
	int			today;
	int			**days_matrix;
	int			current_month_index;
	int			month_index;
	int			days_in_month = 0;
	int			week_day_index;

	system_date = localtime(&timestamp);

	mktime(system_date);

	year = system_date->tm_year;
	current_month_index	= system_date->tm_mon;
	today = system_date->tm_mday;

	days_matrix = (int**)calloc(calendar_size, sizeof(int*));
	for (i=0; i < calendar_size; i++) {
		days_matrix[i] = (int*)calloc(31, sizeof(int));
	}

	for (i=0; i < calendar_size; i++) {

		month_index = ((current_month_index - i) % 12);
		if (month_index == -1) {
			year--;
		}

		if (month_index < 0) {
			month_index = 12 + month_index;
		}

		days_in_month = get_days_in_month(year, month_index);
		first_month_wday = get_first_week_day_of_month(year, month_index);

		for (j=0; j < days_in_month; j++) {

			week_day_index = (j + first_month_wday) % 7;
			days_matrix[i][j] = 1;

			if ((week_day_index == 0) || (week_day_index == 6)) {
				days_matrix[i][j] = 0;
			}
		}

		// mark as invalid day all days over month limit.
		for (j=days_in_month; j < 31; j++) {
			days_matrix[i][j] = -1;
		}
	}

	// mark as invalid days the future.
	for (i=today; i < days_in_month; i++) {
		days_matrix[0][i] = -1;
	}

	return days_matrix;
}

int
apply_calendar_baseline_exceptions(
		long baseline_id,
		int **calendar,
		int calendar_size,
		time_t timestamp)
{
    MYSQL_RES	*res = NULL;
    MYSQL_ROW	row;
    char	*query;
    int		i, num_fields = 0;
    Baseline	baseline_exception;
    int		month_index;
    int		baseline_year;
    int		baseline_month;
    int		baseline_day;
	char    date[32];
	int		current_year;
	int		current_month;
	struct	tm *system_date;

	system_date = localtime(&timestamp);
	timestamp_to_longdate(date, timestamp);

	current_year  = system_date->tm_year + 1900;
	current_month = system_date->tm_mon + 1;

    asprintf(
        &query,
        "SELECT *"
        " FROM baseline_calendar"
        " WHERE date < '%s' AND baseline_id=%lu",
		date,
        baseline_id
    );

	if (do_query(query, DBHOST, DBUSER, DBPASS, "opcfg", &res) <= 0) {
		free(query);
		return OK;
	}

	free(query);

    num_fields = mysql_num_fields(res);

    while ((row = mysql_fetch_row(res))) {
        for (i=0; i < num_fields; i++) {
            if ((row[i] == NULL) || (strlen(row[i]) == 0)) {
				continue;
            }

            switch(i) {
                case FIELD_BASELINE_CALENDAR_ID:
                case FIELD_BASELINE_CALENDAR_ALARM:
                case FIELD_BASELINE_CALENDAR_SAMPLES:
                case FIELD_BASELINE_MULTIPLY_UP:
                case FIELD_BASELINE_MULTIPLY_DOWN:
                    continue;
                case FIELD_BASELINE_CALENDAR_DATE:
                    asprintf(&baseline_exception.date, "%s", row[i]);
                    break;
                case FIELD_BASELINE_CALENDAR_UTIL:
                    baseline_exception.util = atoi(row[i]);
                    break;

            }

        }

        baseline_year	= atoi(strtok(baseline_exception.date, "-"));
        baseline_month	= atoi(strtok(NULL, "-"));
        baseline_day	= atoi(strtok(NULL, "-"));

		free(baseline_exception.date);

        month_index     = ((current_year - baseline_year) * 12);
        month_index	+= (current_month - baseline_month);

        // because C indexes starts at 0.
        baseline_day--;

		if (month_index >= calendar_size) {
			break;
		}

        // if month_index/baseline_day is a valid day
        if (calendar[month_index][baseline_day] >= 0) {
            calendar[month_index][baseline_day] = baseline_exception.util;
        }
    }

    mysql_free_result(res);

    return OK;
}

int**
gen_baseline_calendar(Baseline baseline, time_t timestamp)
{
	int			**calendar;
	struct tm	*system_date;
	int			year;
	int			month;
	int			calendar_size = baseline.samples * 3;

	system_date = localtime(&timestamp);

	mktime(system_date);

	year	= system_date->tm_year + 1900;
	month	= system_date->tm_mon + 1;

	calendar = gen_default_calendar(calendar_size, timestamp);
	apply_calendar_baseline_exceptions(baseline.id, calendar, calendar_size, timestamp);

	return calendar;
}

long
get_hostid_by_hostname(const char *hostname)
{
	MYSQL_RES	*res = NULL;
	MYSQL_ROW	row;
	char		*query;
	long		host_id = 0;
	int			num_fields;

	asprintf(
		&query,
		"SELECT host_id"
		" FROM nagios_hosts"
		" WHERE host_name=\"%s\"",
		hostname
	);

	do_query(query, DBHOST, DBUSER, DBPASS, "opcfg", &res);
	free(query);

	num_fields = mysql_num_fields(res);

	if (num_fields < 1) {
		mysql_free_result(res);
		return host_id;
	}

	if (((row = mysql_fetch_row(res))) && (row[0] != NULL)
		&& (strlen(row[0]) > 0)) {

		host_id = atoi(row[0]);
	}

	mysql_free_result(res);

	return host_id;
}

long
get_serviceid_by_hostid_servicename(long host_id, const char *servicename)
{
	MYSQL_RES	*res = NULL;
	MYSQL_ROW	row;
	char		*query;
	long		service_id = 0;
	int			num_fields;

	asprintf(
		&query,
		"SELECT service_id"
		" FROM nagios_services"
		" WHERE host_id=\"%lu\""
		" 	AND service_description=\"%s\"",
		host_id,
		servicename
	);

	do_query(query, DBHOST, DBUSER, DBPASS, "opcfg", &res);
	free(query);

	num_fields = mysql_num_fields(res);

	if (num_fields < 1) {
		return service_id;
	}

	if (((row = mysql_fetch_row(res))) && (row[0] != NULL)
		&& (strlen(row[0]) > 0)) {

		service_id = atoi(row[0]);
	}

	mysql_free_result(res);

	return service_id;
}

long
get_metricid_by_hostid_serviceid_metricname(long host_id, long service_id,	const char *metricname)
{
	MYSQL_RES	*res = NULL;
	MYSQL_ROW	row;
	char		*query;
	long		metric_id = 0;
	int			num_fields;

	asprintf(
		&query,
		"SELECT metric_id"
		" FROM service_metric"
		" WHERE host_id=\"%lu\""
		" 	AND service_id=\"%lu\""
		" 	AND metric=\"%s\"",
		host_id,
		service_id,
		metricname
	);

	do_query(query, DBHOST, DBUSER, DBPASS, "opcfg", &res);
	free(query);

	num_fields = mysql_num_fields(res);

	if (num_fields < 1) {
		return metric_id;
	}

	if (((row = mysql_fetch_row(res))) && (row[0] != NULL)
		&& (strlen(row[0]) > 0)) {

		metric_id = atoi(row[0]);
	}

	mysql_free_result(res);

	return metric_id;
}

long
get_baselineid(long metric_id)
{
	MYSQL_RES	*res = NULL;
	MYSQL_ROW	row;
	char		*query;
	long		baseline_id = 0;
	int			num_fields;

	asprintf(
		&query,
		"SELECT baseline_id"
		" FROM service_metric"
		" WHERE metric_id=\"%lu\"",
		metric_id
	);

	do_query(query, DBHOST, DBUSER, DBPASS, "opcfg", &res);
	free(query);

	num_fields = mysql_num_fields(res);

	if (num_fields < 1) {
		return baseline_id;
	}

	if (((row = mysql_fetch_row(res))) && (row[0] != NULL)
		&& (strlen(row[0]) > 0)) {

		baseline_id = atoi(row[0]);
	}

	mysql_free_result(res);

	return baseline_id;
}

void
remove_invalid_char(char **orig)
{
	int i, j;
	int size = strlen((*orig));
	char *tmp;

	tmp = (char*)calloc(strlen((*orig)) + 1, sizeof(char));
	strcpy(tmp, (*orig));


	for (i=0, j=0; i < size; i++) {
		if (((*orig)[i] != '\'') && ((*orig)[i] != '"') && ((*orig)[i] !=';')) {
			tmp[j++] = (*orig)[i];
		}
	}

	tmp[j] = '\0';

	bzero((*orig), strlen((*orig)));
	strcpy((*orig), tmp);

	free(tmp);
}
